function [c, ceq, gradc, gradceq] = dp_confn(para,LLS)
% Nonlinear constraint corresponding to gradient of NLLS in Barwick-Pathak
% - LLS.id is the panel unit, consecutive from 1 to max(id);
% - LLS.m is the choice id within the panel unit, consecutive from 1 to 
%   max(m), need not be balanced across units
% - LLS.y is outcome corresponding to chosen choice (0 or 1)
% - LLS.u_x is matrix of covariates corresponding to u
% - LLS.W_x is matrix of covariates corresponding to W
% - LLS.EW_x is matrix of covariates corresponding to EW

c=[];
gradc=[];

Nj=LLS.Nj;				% number of model parameters	
Nk=LLS.Nk;              % number of splines in value function
para_u=para(1:Nj);		% model parameters (utility function)
para_W=para(Nj+1:end);	% spline parameters in value function
delta=LLS.delta;
Nobs=LLS.Nobs;
sumwt=sum(LLS.wt);
assert(size(para,1)==Nj+Nk&&size(para,2)==1, ...
    'Error: para vector incorrect size');
maxm=max(LLS.m);

expv = exp(LLS.u_x*para_u + delta*LLS.EW_x*para_W);
sumexpv=zeros(Nobs,1);
for m=1:maxm
    ids=LLS.id(LLS.m==m);
    sumexpv(ids)=expv(LLS.m==m,:)+sumexpv(ids);
end
f = LLS.W_x*para_W - log(sumexpv);

% Calculate constraint
pr=expv./sumexpv(LLS.id); 
dfjdb = -(repmat(pr,1,Nj).*LLS.u_x);
dWkdt = (repmat(delta*pr,1,Nk).*LLS.EW_x);
dfdb=zeros(Nobs,Nj);
dWdt=zeros(Nobs,Nk);
for m=1:maxm
    ids=LLS.id(LLS.m==m);
    dWdt(ids,:)=dWkdt(LLS.m==m,:)+dWdt(ids,:);
    dfdb(ids,:)=dfjdb(LLS.m==m,:)+dfdb(ids,:);
end
dfdt=LLS.W_x - dWdt;
ceq=dfdt' * (LLS.wt.*f) /sumwt;

% Calculate constraint gradient
gradceq=zeros(Nj+Nk,Nk);
temp1=zeros(Nj,Nk);
for j=1:Nj
    temp2=zeros(Nobs,Nk);
    for m=1:maxm
        u_xj=LLS.u_x(LLS.m==m,j);
        ids=LLS.id(LLS.m==m,:);
        temp2(ids,:)=-repmat(u_xj,1,Nk).*dWkdt(LLS.m==m,:)+temp2(ids,:);
    end
    temp1(j,:)=(LLS.wt.*f)'*temp2;
end;
gradceq((1:Nj),:) = temp1 - (repmat(LLS.wt.*f,1,Nj).*dfdb)'*dWdt ...
    + (repmat(LLS.wt,1,Nj).*dfdb)'*dfdt;

temp1=zeros(Nk,Nk);
for k=1:Nk
    temp2=zeros(Nobs,Nk);
    for m=1:maxm
        EW_xk=LLS.EW_x(LLS.m==m,k);
        ids=LLS.id(LLS.m==m,:);
        temp2(ids,:)=-repmat(EW_xk,1,Nk).*dWkdt(LLS.m==m,:)+temp2(ids,:);
    end
    temp1(k,:)=(LLS.wt.*f)'*delta*temp2;
end;
gradceq(((1+Nj):end),:) = temp1 + (repmat(LLS.wt.*f,1,Nk).*dWdt)'*dWdt ...
    + (repmat(LLS.wt,1,Nk).*dfdt)'*dfdt;

gradceq=gradceq/sumwt;
