/*****************************************************************************************
AUTHOR: David Chan
PURPOSE: Calculate counterfactual value functions under Barwick-Pathak implementation
NOTES: 
- Written to allow looping over several counterfactual assignment functions, each time
  sending data to MATLAB to estimate counterfactual policy functions using utility 
  parameters previously estimated and NLLS
OPTIONS:
- usesimdata: First simulate under counterfactual assignment policy (and observed 
  discharge policy) before preparing BP data for NLLS 
******************************************************************************************/

global shiftlens 7 8 9 10 12
global cf_extend_list -5(1)-2 2(1)5
global nshifttypes_7 9/11
global nshifttypes_8 12/13
global nshifttypes_9 14/35
global nshifttypes_10 1/6
global nshifttypes_12 7/8
global interval=5
global discharge_estimates clogit
global arrival_estimates ologit
global xbd_estimates xbd_lntdc
global estfile ${Data}/estimates_dp
global delta=.98
global keepirelto_range 1 60 
global simirelto_range -60 60

clear all 

*** Prepare data, then export ************************************************************
set more off
foreach shiftlen of numlist $shiftlens {
global shiftlen=`shiftlen'
set_global_knots, shiftlen(`shiftlen')
global infile dp_data_for_est.csv
qui foreach num of numlist $cf_extend_list {
	noi disp as text _n "shiftlen = `shiftlen' ; cf_extend = `num'"
	global cf_extend=`num'
	global estname dp_estimates_l${shiftlen}
	assert ${cf_extend}!=0 
	if sign(${cf_extend})==1 local cfname _cf+${cf_extend}
	else if sign(${cf_extend})==-1 local cfname _cf${cf_extend}
	global cf_estname ${estname}`cfname'
	
	local cfparams cf_from(`=abs(${cf_extend})') cf_extend(${cf_extend})
	local prep_dp_data_opts template(Temporary/sim_int5_d_template) ///
		arrival_estimates(${arrival_estimates}_len${shiftlen}) ///
		xbd_estimates(${xbd_estimates}_len${shiftlen}) ///
		if(shiftlen==${shiftlen}) incensus
	noi disp as text "  Preparing data"

	// Simulate counterfactual arrivals
	tempfile simobs
	local appendsim=0
	local numlist ${nshifttypes_${shiftlen}}
	foreach nshifttype of numlist `numlist' {
		noi disp as text "    nshifttype == `nshifttype': $S_DATE $S_TIME"
		use "${Data}/stored_dp_data.dta", clear
		local opt interval(${interval})
		local opt `opt' template(Temporary/sim_int5_d_template)
		local opt `opt' arrival_estimates(${arrival_estimates}_len${shiftlen}_`nshifttype')
		local opt `opt' xb_lntdc_estimates(${xbd_estimates}_len${shiftlen})
		local opt `opt' prep_dp_data_opts(`prep_dp_data_opts' ///
			nshifttypes(`nshifttype') noleave)
		local opt `opt' ext_estimates_opts(estname(${estname}) estfile(${estfile}))
		local opt `opt' simirelto_range(${simirelto_range}) 
		local opt `opt' dpirelto_range(${keepirelto_range})
		local opt `opt' discharge_estimates(${discharge_estimates}_len${shiftlen}) 
		local opt `opt' noseed `cfparams'
		run_sim if shiftlen==${shiftlen}&nshifttype==`nshifttype', `opt'
		if `appendsim' append using `simobs'
		save `simobs', replace
		local appendsim=1
	}

	prep_dp_data, `prep_dp_data_opts' nshifttypes(${nshifttypes_${shiftlen}}) ///
		noknots keepirelto_range(${keepirelto_range}) `cfparams' nodcpr usesimdata
	if ${shiftlen}==9 {        
		merge m:1 nshiftid using "${Data}/nshiftid_sample_xwalk.dta", ///
			keep(match) nogen
		rename groupid oldgroupid
		egen groupid=group(oldgroupid)
		drop oldgroupid	
	}

	proc_dp_varlist, prep(eos(W_incensus_sp1) w(W_incensus_sp1 ///
		W_diff_lntdc_sp1 W_eos*) ///
		w1(W_incensus_sp1) w2(W_adjirelto_sp1) ///
		u(u_diff_lntdc_sp? u_xbd_lntdc_sp1)) estname(${varver})
	ext_estimates, estname(${estname}) estfile(${estfile}) assert(${varver})
		
	local en ${varver}_
	global vars ${`en'u_vars} ${`en'w_vars} ${`en'Ew_vars} ${`en'h_vars} ${`en'Eh_vars}
	sort groupid sb
	by groupid: gen choicenum=_n
	gen byte eos=irelto==1
	keep groupid sb wt choice choicenum eos ${vars}
	order groupid sb wt choice choicenum eos ${vars}
	outsheet using "${Data}/${infile}", comma replace

	ext_estimates, estname(${estname}) estfile(${estfile}) ///
		export(${Output}/ext_results.txt) nodisp
	assert scalar(delta)==${delta}
	
	noi disp as text "  Estimating counterfactuals in MATLAB"
	capture erase "${Output}/results.txt"
	local cmd cd Code;
	local cmd `cmd' dp_main_fun('${infile}','results.txt','delta',`=scalar(delta)',
	local cmd `cmd' 'parafile','ext_results.txt','cf',true,'eoscon',true,
	local cmd `cmd' 'eosvar','eos')
	!"${callmatlab}" -nodisplay -nosplash -nodesktop -r "`cmd'; exit;" 
	
    capture confirm file "${Output}/results.txt"
    while _rc {     
        sleep 1000 
        capture confirm file "${Output}/results.txt"
    }
	
	ext_estimates, estname(${cf_estname}) estfile(${estfile}) ///
		useresults(delta(`=scalar(delta)') varver(${varver}) ///
		file(${savedir}/Results/results.txt)) append replace
}
}
