/*****************************************************************************************
AUTHOR: David Chan
PURPOSE: Create Figure A-1.1
NOTES: 
- To be run from master_tables_figures.do
- May run individually, but need to run preamble in master_tables_figures.do first
- Results are identical to those in Tables 2 and 3, but are presented in graphical form
*****************************************************************************************/

global controlvar i.visit_dow i.nvisit_ym i.ed_hh age age2 male female35 de? ///
	v? v?? dr? dl? p? triage_ed i.irelfrom 
global controlvar2a age age2 male female35 de? dr? dl?
global controlvar2b age age2 male female35 de? dr? dl? v? v??

capture program drop predict_partial
program predict_partial
syntax , gen(string)
	qui foreach var of varlist visit_dow nvisit_ym ed_hh irelfrom irelto {
		gen `var'_orig=`var'
		sum `var'
		replace `var'=r(min)
	}
	qui foreach var of varlist p? {
		gen `var'_orig=`var'
		replace `var'=0
	}
	capture drop `gen'
	predict `gen', xb
	qui foreach var of varlist visit_dow nvisit_ym ed_hh irelfrom p? irelto {
		replace `var'=`var'_orig
		drop `var'_orig
	}
end

*** Load data ****************************************************************************
gen_working_data, poe(sum)
gen_var, visittime elix control outcomes team dem_vit costs
gen_reltimeext, by(pod npid) posthrs(6) keepmaster
gen irelto=ceil(hours(shiftend_tc-ed_tc))
replace irelto=7 if irelto>7&irelto!=.
gen relfrom=hours(ed_tc-shiftbeg_tc)
gen irelfrom=ceil(relfrom)+6 // add 6 since factor variables cannot have negative numbers

*** Run regressions **********************************************************************
// Identical to Tables 2 (Column 5) and 3 
tempfile master estfile
save `master', replace
qui foreach outcome in lntdc admit lntotcost all_c_all {
	use `master', clear
	noi disp as text "`outcome'"
	areg `outcome' ib(last).irelto ${controlvar}, absorb(team)
	regsave
	keep var coef stderr
	rename coef coeff 
	rename stderr se
	keep if regexm(var,"[1-6]\.irelto")
	replace var=regexr(var,"\.irelto","")
	gen outcome="`outcome'"
	destring var, force gen(irelto)
	capture append using `estfile'
	save `estfile', replace
}

local control_xb ib(last).irelto i.visit_dow i.nvisit_ym i.ed_hh p? i.irelfrom
qui foreach outcome in lntdc admit lntotcost all_c_all {
	foreach suffix in 2a 2b {
		noi disp as text "xb`suffix'_`outcome'"
		use `master', clear
		areg `outcome' ${controlvar`suffix'} `control_xb', absorb(npid)
		predict_partial, gen(xb`suffix'_`outcome')
		areg xb`suffix'_`outcome' `control_xb', absorb(team)
		regsave
		keep var coef stderr
		rename coef coeff 
		rename stderr se
		keep if regexm(var,"[1-6]\.irelto")
		replace var=regexr(var,"\.irelto","")
		gen outcome="xb`suffix'_`outcome'"
		destring var, force gen(irelto)
		capture append using `estfile'
		save `estfile', replace		
	}
}

*** Plot graph **************************************************************************
replace irelto=-abs(irelto)
foreach outcome in lntdc admit lntotcost all_c_all {
	local ytitle_lntdc Log (hours) length of stay
	local ytitle_admit Admission probability
	local ytitle_lntotcost Log total direct costs
	local ytitle_all_c_all Number of orders
	local title_lntdc A: Length of Stay
	local title_all_c_all B: Orders
	local title_admit C: Inpatient Admission
	local title_lntotcost D: Costs
	twoway (line coeff irelto if outcome=="`outcome'", lcolor(black) lpattern(dash)) ///
		(connected coeff irelto if outcome=="xb2b_`outcome'", mcolor(black) msymbol(O) ///
		lcolor(black)) (connected coeff irelto if outcome=="xb2a_`outcome'", mcolor(black) ///
		msymbol(Oh) lpattern(shortdash) lcolor(black)), ///
		legend(off) aspectratio(1) xtitle("Hours relative to EOS") ///
		ytitle("`ytitle_`outcome''") title("`title_`outcome''", size(medlarge) color(black)) ///
		name(`outcome', replace) graphregion(color(white)) ylabel(, nogrid)
}		
			
graph combine lntdc all_c_all admit lntotcost, ysize(6) iscale(*.9) graphregion(color(white))
graph export "${Output}/Figure_A-1.1.eps", as(eps) replace	
