/*****************************************************************************************
AUTHOR: David Chan
PURPOSE: Create Figure A-1.2
NOTES: 
- To be run from master_tables_figures.do
- May run individually, but need to run preamble in master_tables_figures.do first
*****************************************************************************************/

global begtime clock("1jun2005 00:00","DMYhm")
global controlvar2a age_r age2_r male_r female35_r de?_r dr?_r dl?_r
global controlvar2b age_r age2_r male_r female35_r de?_r dr?_r dl?_r v?_r v??_r
global controlvar_comp i.visit_dow i.nvisit_ym i.ed_hh triage_ed i.irelfrom 
global controlvar i.visit_dow i.nvisit_ym i.ed_hh age age2 male female35 de? ///
	v? v?? dr? dl? p? triage_ed i.irelfrom 

*** Program to extract i.irelto beta's ***************************************************
capture program drop extract_irelto_b
program extract_irelto_b
syntax , gen(string) 
qui {
	mat b=e(b)
	capture drop `gen'
	gen `gen'=.
	foreach num of numlist 1/7 {
		replace `gen'=b[1,`num'] if irelto==`num'
	}
}
end

*** Load data ****************************************************************************
gen_working_data, poe(sum)
gen_var, visittime elix control outcomes team dem_vit
gen_reltimeext, by(pod npid) posthrs(6) keepmaster

capture drop at_t
gen at_t=ceil(minutes(ed_tc-${begtime})/60)

// Other variables
gen irelto=ceil(hours(shiftend_tc-ed_tc))
replace irelto=7 if irelto>7&irelto!=.
gen relfrom=hours(ed_tc-shiftbeg_tc)
gen irelfrom=ceil(relfrom)+6 // add 6 since factor variables cannot have negative numbers

capture drop nshiftid
encode shiftid, gen(nshiftid) // For jack-knife specifications
sum nshiftid

*** Generate jack-knife coefficients ******************************************************
tempfile pre_jk_coef
save `pre_jk_coef', replace
keep lntdc irelto visit_dow nvisit_ym ed_hh age age2 male female35 de? v? v?? dr? dl? ///
	p? triage_ed irelfrom npid nshiftid

save "${Data}/temp_data_fig_a-1.2.dta", replace	
use "${Data}/temp_data_fig_a-1.2", clear
global savefile irelto_coeff
set more off
local nshiftid=1
qui while `nshiftid'<22501 {
	noi disp as text "nshiftid = `nshiftid'"
	preserve
	qui areg lntdc ib(last).irelto ${controlvar} if nshiftid!=`nshiftid', absorb(npid)
	clear
	set obs 6
	gen irelto=_n
	extract_irelto_b, gen(coeff)
	gen nshiftid=`nshiftid'
	append using "${Data}/${savefile}"
	save "${Data}/${savefile}", replace
	restore
	local nshiftid=`nshiftid'+1
}
use `pre_jk_coef', clear

*** Residualize **************************************************************************
// Extract "value added" of irelto for each observation 
extract_irelto_b, gen(Q_sb)
by at_t, sort: egen Q_hr=mean(Q_sb)

// Or use jack-knifed coefficients
merge m:1 nshiftid irelto using "${Data}/temp_data_fig_a-1.2", nogen ///
	keep(master match)
by at_t, sort: egen Q_hr_jk=mean(coeff)
	
// Generate residualized LOS
// Absorb npid so residuals will be more meaningful (otherwise, all teams with only 1
// observation will have residual=0)
// Average out within-hour variation
predict resid_lntdc, resid
by at_t, sort: egen lntdc_hr=mean(resid_lntdc+Q_sb)

*** Generate predicted LOS based on patient characteristics ******************************
capture drop *xb*
keep lntdc *_hr* visit_dow nvisit_ym ed_hh age age2 male female35 de? ///
	v? v?? dr? dl? p? triage_ed irelfrom sb at_t npid irelto nshiftid
disp as text "Variables: " _c
qui foreach var of varlist age age2 male female35 de? dr? dl? v? v?? {
	noi disp as text "`var' " _c
	areg `var' ${controlvar_comp}, absorb(npid)
	predict `var'_r, resid
}
disp _n
foreach suffix in 2a 2b {
	qui areg lntdc ${controlvar`suffix'}, absorb(irelto) 
	predict xb_lntdc_`suffix', xb
	by at_t, sort: egen lntdc_xb`suffix'_hr=mean(xb_lntdc_`suffix')
}

*** Regressions **************************************************************************
{/*
// Jack-knifed coefficients
. reg lntdc_hr Q_hr_jk, vce(cluster at_t)

Linear regression                                      Number of obs =  409352
                                                       F(  1, 63344) =  294.37
                                                       Prob > F      =  0.0000
                                                       R-squared     =  0.0065
                                                       Root MSE      =  .33524

                               (Std. Err. adjusted for 63345 clusters in at_t)
------------------------------------------------------------------------------
             |               Robust
    lntdc_hr |      Coef.   Std. Err.      t    P>|t|     [95% Conf. Interval]
-------------+----------------------------------------------------------------
     Q_hr_jk |   1.029458   .0600015    17.16   0.000      .911855    1.147061
       _cons |   .0013511   .0019375     0.70   0.486    -.0024463    .0051485
------------------------------------------------------------------------------

*** Omit i.irelto in generating *_r variables ********************************************

. reg lntdc_xb2a_hr Q_hr, vce(cluster at_t)

Linear regression                                      Number of obs =  409375
                                                       F(  1, 63354) =    1.37
                                                       Prob > F      =  0.2424
                                                       R-squared     =  0.0000
                                                       Root MSE      =  .15344

                               (Std. Err. adjusted for 63355 clusters in at_t)
------------------------------------------------------------------------------
             |               Robust
lntdc_xb2a~r |      Coef.   Std. Err.      t    P>|t|     [95% Conf. Interval]
-------------+----------------------------------------------------------------
        Q_hr |   .0294256   .0251732     1.17   0.242    -.0199139    .0787652
       _cons |   1.050762    .000878  1196.82   0.000     1.049041    1.052483
------------------------------------------------------------------------------

. reg lntdc_xb2b_hr Q_hr, vce(cluster at_t)

Linear regression                                      Number of obs =  409375
                                                       F(  1, 63354) =    0.88
                                                       Prob > F      =  0.3487
                                                       R-squared     =  0.0000
                                                       Root MSE      =  .15705

                               (Std. Err. adjusted for 63355 clusters in at_t)
------------------------------------------------------------------------------
             |               Robust
lntdc_xb2b~r |      Coef.   Std. Err.      t    P>|t|     [95% Conf. Interval]
-------------+----------------------------------------------------------------
        Q_hr |   .0242289   .0258552     0.94   0.349    -.0264474    .0749052
       _cons |    1.05065   .0008987  1169.08   0.000     1.048889    1.052412
------------------------------------------------------------------------------
*/}

*** Plot results *************************************************************************
by at_t, sort: egen sbcount=count(_n)
keep sbcount *_hr at_t
duplicates drop
sort Q_hr // at_t // include at_t because of ties in Q_hr
gen sumsbcount=sum(sbcount)
gen vigintile=ceil(sumsbcount/sumsbcount[_N]*20)
qui foreach var in Q lntdc lntdc_xb2a lntdc_xb2b {	
	gen `var'_v=.
	foreach vigintile of numlist 1/20 {
		sum `var'_hr [aw=sbcount] if vigintile==`vigintile'
		replace `var'_v=r(mean) if vigintile==`vigintile' 
	}	
}
keep vigintile *_v
duplicates drop
foreach var in Q lntdc lntdc_xb2a lntdc_xb2b {
	egen mean_`var'_v=mean(`var'_v)
	replace `var'_v=`var'_v-mean_`var'_v
}
drop mean*

/* Omit i.irelto in generating *_r variables; use jack-knife Q_hr
vigintile	Q_v	lntdc_v	lntdc_xb2a_v	lntdc_xb2b_v
1	-.0771694	-.0808855	.0019096	.0028208
2	-.0384886	-.0367295	-.0023041	-.0011257
3	-.027781	-.0342201	.0026916	.0028878
4	-.0195889	-.0234092	.0023615	.0022171
5	-.0134673	-.0134478	-.0060538	-.0063227
6	-.008516	-.0035718	-.0091633	-.0104547
7	-.0039995	-.0004033	-.0077395	-.0084678
8	-9.62e-06	-.0060484	-.0036527	-.0034914
9	.0025051	.0044065	-.0013659	-.0019455
10	.0056207	.0141154	-.0015292	-.0008991
11	.0084403	.0056188	-.003304	-.0034055
12	.0110347	.0066629	.0008684	.0010071
13	.0135498	.0169738	-.0032469	-.0031859
14	.0158907	.0014661	.0015599	.0023656
15	.0176092	.0073287	.0054352	.0052154
16	.0195813	.0245535	.0004618	.0003999
17	.0222573	.013464	-.0054474	-.0055704
18	.0241771	.0298453	.0103054	.0107751
19	.0241771	.0384953	.0070212	.0070391
20	.0241771	.0357855	.0111914	.0101397
*/

capture rename Q_v q_v
foreach var of varlist lntdc_v lntdc_xb2a_v lntdc_xb2b_v {
	qui reg `var' q_v
	predict pred_`var', xb
}
twoway (line pred_lntdc_v q_v, lcolor(black)) (scatter lntdc_v q_v, mcolor(black)), ///
	legend(off) aspectratio(1.1) xtitle("Predicted by EOS") ///
	ytitle("Actual") ylabel(-.1(.05).1) ///
	title("A: Actual", size(medlarge) color(black)) ///
	name(lntdc, replace) graphregion(color(white)) ylabel(, nogrid) ///
	note("Coeff = 1.029 (.060)", position(5) ring(0))
twoway (line pred_lntdc_xb2a_v q_v, lcolor(black)) (scatter lntdc_xb2a_v q_v, mcolor(black)), ///
	legend(off) aspectratio(1.1) xtitle("Predicted by EOS") ///
	ytitle("Predicted by characteristics") ylabel(-.1(.05).1) ///
	title("B: Ex Ante Characteristics", size(medlarge) color(black)) ///
	name(lntdc_xb2a, replace) graphregion(color(white)) ylabel(, nogrid) ///
	note("Coeff = 0.029 (.025)", position(5) ring(0))
twoway (line pred_lntdc_xb2b_v q_v, lcolor(black)) (scatter lntdc_xb2b_v q_v, mcolor(black)), ///
	legend(off) aspectratio(1.1) xtitle("Predicted by EOS") ///
	ytitle("Predicted by characteristics") ylabel(-.1(.05).1) ///
	title("C: All Characteristics", size(medlarge) color(black)) ///
	name(lntdc_xb2b, replace) graphregion(color(white)) ylabel(, nogrid) ///
	note("Coeff = .024 (.026)", position(5) ring(0))
graph combine lntdc lntdc_xb2a lntdc_xb2b, ysize(2) xsize(5) rows(1) iscale(*1.6) ///
	graphregion(color(white))
graph export "${Output}/Figure_A-1.2.eps", as(eps) replace	
