/*****************************************************************************************
AUTHOR: David Chan
PURPOSE: Create Figure A-5.2
NOTES: 
- To be run from master_tables_figures.do
- May run individually, but need to run preamble in master_tables_figures.do first
*****************************************************************************************/

set seed 0
global interval=5
global shiftlens 7 8 9 10 12
global arrival_estimates ologit
global discharge_estimates clogit
global estfile ${Data}/estimates_dp
global simirelto_range -60 108
global nshifttypes_7 9/11
global nshifttypes_8 12/13
global nshifttypes_9 14/35
global nshifttypes_10 1/6
global nshifttypes_12 7/8

capture program drop sim_arrival_model 
program sim_arrival_model
syntax if, estfile(string) [exact]
qui {
	noi disp as text "Simulating for sample: `if'"	
	keep `if'
	estimates use "${Estimates}/`estfile'"
	make_spline, retrieve estfile(`estfile')
	predict p1 p2 p3 p4
	gen rand=runiform()
	if "`exact'"=="exact" gen sim_visits=p2*1+p3*2+p4*3
	else gen sim_visits=cond(rand<p1,0,cond(rand<p1+p2,1,cond(rand<p1+p2+p3,2,3)))
	drop p1 p2 p3 p4 
}
end

*** Generate and plot simulated arrivals *************************************************
// Hold discharges fixed
use "${Data}/arrival_count_data", clear

// Simulate arrivals
keep if irelfrom_base>0
tempfile master sim_obs
save `master', replace
local run=1
qui foreach shiftlen in $shiftlens {
	levelsof nshifttype if shiftlen==`shiftlen'
	local levels `r(levels)'
	foreach nshifttype in `levels' {
		use `master', clear
		local estfile ${arrival_estimates}_len`shiftlen'_`nshifttype'
		noi sim_arrival_model if shiftlen==`shiftlen'&nshifttype==`nshifttype', ///
			estfile(`estfile') exact
		if `run'>1 append using `sim_obs'
		save `sim_obs', replace
		local run=`run'+1
	}
}
gen hhr_irelto_base=ceil(irelto_base*${interval}/30)
keep if sim_visits!=.
foreach var of varlist *visits {
	by hhr_irelto_base, sort: egen hhr_`var'=mean(`var')
}
keep hhr_*
duplicates drop
sort hhr_irelto_base

/*
hhr_irelto_base	hhr_visits	hhr_sim_visits
1	0.0073778	0.0045145
2	0.0223634	0.0252268
3	0.0854826	0.0982318
4	0.2234879	0.2136856
5	0.32521	0.3079254
6	0.3805334	0.370752
7	0.4080174	0.4110479
8	0.4149981	0.4436735
9	0.4631317	0.466434
10	0.5099068	0.4914726
11	0.5027171	0.5027589
12	0.5040756	0.5140451
13	0.5189567	0.5201271
14	0.5433683	0.525185
15	0.5092379	0.5175563
16	0.4554404	0.4974501
17	0.4712375	0.4727843
18	0.441158	0.3886706
*/
keep if hhr_irelto_base<=18
gen crelto=-hhr_irelto_base/2
replace hhr_visits=hhr_visits*4
replace hhr_sim_visits=hhr_sim_visits*4
twoway (connected hhr_visits crelto, lcolor(black) mcolor(black)) ///
	(connected hhr_sim_visits crelto, lcolor(black) mcolor(black) lpattern(dash) msymbol(Oh)), ///
	legend(off) aspectratio(1) xtitle("Hours relative to EOS") ///
	ytitle("Patients per hour") title("A: Assignments", size(medlarge) color(black)) ///
	xlabel(-9(3)0) name(arrivals, replace) graphregion(color(white)) ylabel(, nogrid) 

*** Generate simulated discharges ********************************************************
// Hold arrivals fixed with -cfonly- option
foreach model in static dynamic {
	if "`model'"=="static" local dpirelto_range 1 0
	else local dpirelto_range 1 60
	tempfile simobs
	local appendsim=0
	qui foreach shiftlen of numlist $shiftlens {
		global estname dp_estimates_l`shiftlen'_a
		set_global_knots, shiftlen(`shiftlen')
		local numlist ${nshifttypes_`shiftlen'}
	qui foreach nshifttype of numlist `numlist' {
		noi disp as text "nshifttype == `nshifttype': $S_DATE $S_TIME" _n
		use "${Data}/stored_dp_data.dta", clear
		local opt interval(${interval})
		local opt `opt' template(${Data}/sim_int5_d_template)
		local opt `opt' simirelto_range(${simirelto_range})
		local opt `opt' dpirelto_range(`dpirelto_range') 
		local opt `opt' discharge_estimates(${discharge_estimates}_len`shiftlen') 
		local opt `opt' arrival_estimates(${arrival_estimates}_len`shiftlen'_`nshifttype')
		local opt `opt' cf_from(0) cf_extend(0) cfonly
		local opt `opt' prep_dp_data_opts( if(shiftlen==`shiftlen') ///
			template(${Data}/sim_int5_d_template) nshifttypes(`nshifttype') ///
			arrival_estimates(ologit_len`shiftlen') ///
			xbd_estimates(xbd_lntdc_len`shiftlen') ///
			incensus)
		local opt `opt' ext_estimates_opts(estname(${estname}) estfile(${estfile}))	
		noi run_sim if shiftlen==`shiftlen'&nshifttype==`nshifttype', `opt'
		if `appendsim' append using `simobs'
		save `simobs', replace
		local appendsim=1
	}
	}
	sum_sim_data, interval(30)
	gen irelto_hr=-int_irelto_base/2
	save "${Data}/moments_`model'", replace
}

*** Combine data and plot figure ********************************************************
use "${Data}/moments_static", clear
foreach var of varlist sim_* {
	rename `var' `=regexr("`var'","sim_","static_")'
}
merge 1:1 int_irelto_base using "${Data}/moments_dynamic", assert(match) nogen
foreach var of varlist sim_* {
	rename `var' `=regexr("`var'","sim_","dynamic_")'
}
keep if int_irelto_base<=18
foreach outcome in t_ed_dcord lntdc avgcensus {
	local title_t_ed_dcord B: Length of Stay
	local title_lntdc C: Length of Stay
	local title_avgcensus D: Census
	local ytitle_t_ed_dcord Hours
	local ytitle_lntdc Log hours
	local ytitle_avgcensus Patients
	twoway (connected real_`outcome' irelto_hr, mcolor(black) lcolor(black)) ///
		(connected static_`outcome' irelto_hr, mcolor(black) lcolor(black) ///
		lpattern(dash) msymbol(Oh)) (connected dynamic_`outcome' irelto_hr, ///
		mcolor(black) lcolor(black) lpattern(dash) msymbol(Th)), legend(off) aspectratio(1) ///
		xtitle("Hours relative to EOS") title("`title_`outcome''", ///
		size(medlarge) color(black)) ytitle("`ytitle_`outcome''") xlabel(-9(3)0) ///
		name(`outcome', replace) graphregion(color(white)) ylabel(, nogrid)
}
graph combine arrivals t_ed_dcord lntdc avgcensus, ysize(6) iscale(*.9) ///
	graphregion(color(white)) 
graph export "${Output}/Figure_A-5.2.eps", as(eps) replace	
