/*****************************************************************************************
AUTHOR: David Chan
PURPOSE: Create Figures A-6.1 and A-6.2
NOTES: 
- To be run from master_tables_figures.do
- May run individually, but need to run preamble in master_tables_figures.do first
*****************************************************************************************/

*** Short program to merge provider fixed effects from crosswalk *************************	
capture program drop gen_var_fe
program define gen_var_fe
syntax , npid(string) gen(string)
qui {
	tempfile master
	save `master', replace
	use "${Data}/npid_fe_xwalk", clear
	rename npid `npid'
	rename estimate `gen'fe
	merge 1:m `npid' using `master', keep(match using) nogen
}
end	
	
*** Identify peer transitioning near EOS *************************************************
use "${Data}/schedule_data", clear
name_pods, time(date)
cat_shift_types
assert totc>fromtc if totc!=.
assert subtotc>subfromtc if subtotc!=.
drop if inlist(shift,"oncall","admin")
sort pod fromtc subfromtc
keep pod nshiftid npid date *tc
tempfile orig appended
save `orig', replace
foreach num of numlist 1/3 {
	use `orig', clear
	by pod: gen byte eff_postovl=hours(totc-fromtc[_n+`num'])
	by pod: gen postnshiftid=nshiftid[_n+`num']
	by pod: gen postnpid=npid[_n+`num']
	gen byte index=`num'
	drop if eff_postovl>6|eff_postovl<-1
	capture append using `appended'
	save `appended', replace
}

drop if subtotc!=.&subtotc!=totc
sort pod fromtc subfromtc index
sort nshiftid eff_postovl
by nshiftid: drop if eff_postovl>0&eff_postovl[_n-1]>0&eff_postovl[_n-1]!=.
by nshiftid: keep if _n==1
keep nshiftid eff_postovl postnshiftid postnpid
tempfile postnpid_xwalk
save `postnpid_xwalk', replace

*** Create analytical sample *************************************************************
global controlvar_short i.visit_dow i.nvisit_ym i.ed_hh age age2 male female35 de? ///
	v? v?? dr? dl? p? triage_ed 

gen_working_data, poe(sum)
gen_var, visittime elix control outcomes team dem_vit
gen_reltimeext, by(pod npid) posthrs(6) keepmaster
discretize_dta, interval(15) trim
merge m:1 nshiftid using `postnpid_xwalk', keep(master match) nogen
gen_census, of(npid) at_from(ed) at_to(dcord) match interval(15) gens(avg)

gen lntdc_ptadj=ln(t_ed_dcord/avgcensus)
gen irelto_orig=ceil(hours(shiftend_tc-ed_tc))
gen irelto=cond(irelto_orig!=.,min(irelto_orig,7),.)
gen irelfrom=ceil(cond(ed_tc>shiftbeg_tc,hours(ed_tc-shiftbeg_tc),.))
replace irelfrom=13 if irelfrom>13&irelfrom!=.

mkspline from=irelfrom, knots(1 3 5 7) cubic

*** Attending characteristics ************************************************************
rename npid attgnpid
merge_npid_char, npid(attgnpid) gen(attg)
merge_npid_char, npid(postnpid) gen(post)
gen_var_fe, npid(attgnpid) gen(attg)
gen_var_fe, npid(postnpid) gen(post)
gen byte attgmale=attgphyssex=="M"
gen attgage=(visit_td-attgdob_td)/365
sum attgage
gen byte attgold=attgage>r(mean)
gen tenure=(visit_td-attgstart_td)/365 // tenure in years
sum tenure
gen byte attgexpert=tenure>r(mean)
sum attgfe
gen byte attghiprod=cond(attgfe!=.,attgfe<r(mean),.)

gen byte pr_smsex=(postphyssex==attgphyssex)
gen byte pr_older=(attgstart_td-poststart_td)>1*365
gen byte pr_higherprod=(postfe<attgfe&attgfe!=.)
gen_cumhx_peer_mod, npid(attgnpid) peernpid(postnpid) use(${Data}/cumhx_xwalk)
gen byte pr_familiar=cond(impcumhx==.,.,impcumhx>60)
foreach var of varlist pr_* {
	replace `var'=2 if postnpid==.
}
label define prvalues 0 "False" 1 "True" 2 "No peer", replace
label values pr_* prvalues

*** Regressions **************************************************************************
qui foreach var in male old expert hiprod {
	areg lntdc_ptadj i.attg`var'#ib(last).irelto c.from? ${controlvar_short}, absorb(team)
	estimates store `var'
	test 0.attg`var'#1.irelto=1.attg`var'#1.irelto
	scalar p_`var'=r(p)
}
qui foreach var in smsex older familiar higherprod {
	areg lntdc_ptadj i.pr_`var'#ib(last).irelto i.pr_`var' c.from? ///
		${controlvar_short}, absorb(team)
	estimates store `var'
	test 0.pr_`var'#1.irelto=1.pr_`var'#1.irelto
	scalar p_`var'=r(p)
}

*** Plot graphs **************************************************************************
tempfile estfile
qui foreach var in male old expert hiprod {
	estimates restore `var'
	regsave
	keep var coef stderr
	rename coef coeff_`var'
	rename stderr stderr_`var'
	keep if regexm(var,"[1-6•]b?\.irelto")
	replace var=regexr(var,"b?\.irelto","")
	gen haschar=substr(var,1,1)
	gen irelto=substr(var,-1,1)
	drop var
	destring irelto, force replace
	destring haschar, force replace
	capture merge 1:1 haschar irelto using `estfile', assert(match) nogen
	save `estfile', replace
}
/* 
haschar	irelto	coeff_male	stderr_male	coeff_old	stderr_old	coeff_expert	stderr_expert	coeff_hiprod	stderr_hiprod
0	1	-0.163127	0.0952904	-0.18645	0.0666918	-0.1825142	0.0695926	-0.241205	0.0655994
0	2	0.0076307	0.0363447	-0.0128682	0.0310524	-0.0105098	0.0313452	0.0131091	0.0314487
0	3	0.0424169	0.0274233	0.0519609	0.0251484	0.0499924	0.025351	0.0521976	0.025472
0	4	0.0202035	0.0233648	0.0090507	0.0211893	-0.0001451	0.021413	0.0253213	0.0214765
0	5	0.0421413	0.0203141	0.0282853	0.0182446	0.0244914	0.0184107	0.0354265	0.0184858
0	6	-0.00523	0.0163575	-0.0060105	0.0139235	-0.0163475	0.0141754	-0.003341	0.0142518
1	1	-0.1837852	0.0609283	-0.1359858	0.075871	-0.1774569	0.0728833	-0.0731758	0.0814198
1	2	-0.0033385	0.0301645	0.0278169	0.0333429	0.0092137	0.0326913	-0.0244527	0.0333802
1	3	0.0414475	0.0247855	0.0419283	0.0264724	0.0274299	0.0258782	0.0264572	0.0265679
1	4	0.0048024	0.0208461	0.0216355	0.0225374	0.0182088	0.0218751	-0.0146753	0.0225072
1	5	0.0164113	0.0178086	0.0255121	0.0193185	0.0207608	0.0187935	0.0008006	0.0194454
1	6	-0.0042159	0.013539	0.0027316	0.0153908	0.0087375	0.014775	-0.0078213	0.0154202
*/
replace irelto=-abs(irelto)
foreach char in male old expert hiprod {
local title_male A: Male
local title_old B: Older
local title_expert C: Higher Tenure
local title_hiprod D: Faster
twoway (connected coeff_`char' irelto if haschar, lcolor(black) mcolor(black)) ///
	(connected coeff_`char' irelto if !haschar, lcolor(black) mcolor(black) lpattern(dash) ///
	msymbol(Oh)), legend(off) aspectratio(1) xtitle("Hours relative to EOS") ///
	ytitle("Workload-adjusted log LOS") title("`title_`char''", size(medlarge) ///
	color(black)) name(`char', replace) graphregion(color(white)) ///
	ylabel(, nogrid) note("{it:p} = `=string(p_`char',"%9.4g")'", position(5) ring(0))
}	
graph combine male old expert hiprod, ysize(6) iscale(*.9) graphregion(color(white)) ///
	ycommon
graph export "${Output}/Figure_A-6.1.eps", as(eps) replace	

tempfile estfile
qui foreach var in smsex higherprod older familiar {
	estimates restore `var'
	regsave
	keep var coef stderr
	rename coef coeff_`var'
	rename stderr stderr_`var'
	keep if regexm(var,"[1-6•]b?\.irelto")
	replace var=regexr(var,"b?\.irelto","")
	gen haschar=substr(var,1,1)
	gen irelto=substr(var,-1,1)
	drop var
	destring irelto, force replace
	destring haschar, force replace
	capture merge 1:1 haschar irelto using `estfile', assert(match) nogen
	save `estfile', replace
}
/*
haschar	irelto	coeff_smsex	stderr_smsex	coeff_higherprod	stderr_higherprod	coeff_older	stderr_older	coeff_familiar	stderr_familiar
0	1	-0.1950996	0.0879446	-0.1742837	0.0788882	-0.1815746	0.0759703	-0.3673049	0.0785017
0	2	0.0347817	0.034897	0.019144	0.0328057	0.0200057	0.0327619	-0.0174785	0.0359645
0	3	0.0723826	0.0272348	0.0866717	0.0263166	0.0646273	0.0261551	0.0670225	0.0290508
0	4	0.0501367	0.0235081	0.0312773	0.0225418	0.0293976	0.0223618	0.0298664	0.0243792
0	5	0.0584035	0.0206917	0.0282596	0.0195765	0.030975	0.0194391	0.0373116	0.0209343
0	6	0.0197889	0.0168891	0.0090016	0.0156991	0.0060743	0.0155445	0.0109969	0.0163429
1	1	-0.2925247	0.0725572	-0.329625	0.0789003	-0.3395625	0.0822994	-0.1758705	0.0854063
1	2	0.0042717	0.0322839	0.0188582	0.034105	0.0153448	0.0342018	0.047056	0.0376439
1	3	0.0555974	0.0261433	0.037295	0.0269909	0.0613058	0.0272734	0.0510746	0.0298027
1	4	0.0080668	0.0223635	0.0205204	0.0232497	0.0204703	0.0235525	0.032516	0.0250536
1	5	0.0242414	0.0193922	0.0522951	0.0204304	0.0485333	0.0206391	0.056368	0.0215129
1	6	0.0020698	0.0155396	0.0106636	0.0166311	0.0142345	0.0169	0.021138	0.0173533
2	1	0.0845408	0.102123	0.0891492	0.1021206	0.0873286	0.1021303	0.0682314	0.1040026
2	2	-0.0782993	0.0416847	-0.0755677	0.0416914	-0.0767132	0.0416997	-0.0902605	0.0446185
2	3	-0.0329954	0.0301049	-0.0308286	0.0301105	-0.0321689	0.0301146	-0.0402921	0.0325939
2	4	-0.034844	0.0245926	-0.0340732	0.024593	-0.0348061	0.0245958	-0.0334491	0.0261779
2	5	-0.0160556	0.0212994	-0.0157262	0.0212992	-0.0163067	0.0213026	-0.0106443	0.0223254
2	6	-0.0378742	0.016984	-0.0376216	0.016983	-0.0379483	0.0169847	-0.0374121	0.0174842
*/
replace irelto=-abs(irelto)
foreach char in smsex older familiar higherprod {
local title_smsex A: Same Sex
local title_older B: Peer Higher Tenure
local title_familiar C: Familiar
local title_higherprod D: Peer Faster
twoway (connected coeff_`char' irelto if haschar==1, lcolor(black) mcolor(black)) ///
	(connected coeff_`char' irelto if haschar==0, lcolor(black) mcolor(black) ///
	msymbol(Oh)), legend(off) aspectratio(1) xtitle("Hours relative to EOS") ///
	ytitle("Workload-adjusted log LOS") title("`title_`char''", size(medlarge) ///
	color(black)) name(`char', replace) graphregion(color(white)) ///
	ylabel(, nogrid) note("{it:p} = `=string(p_`char',"%9.4f")'", position(5) ring(0))
}	
graph combine smsex older familiar higherprod, ysize(6) iscale(*.9) graphregion(color(white)) ///
	ycommon
graph export "${Output}/Figure_A-6.2.eps", as(eps) replace
