/*****************************************************************************************
AUTHOR: David Chan
PURPOSE: Prepare counterfactual simulation data, collecting summary statistics and
regression coefficients
NOTES: 
- _supp indicates that simulated observations are augmented with real observations from
  shifts with unknown types (i.e., we do not know the overlap or length of these shifts)
- const_b indicates adjustment for time relative to beginning of shift, where coefficients
  on from? splines are from regressions in make_table_a-7.3.do (Table A-7.3)
******************************************************************************************/

global rep_list 1/5
global cf_extend_list -5(.5)-1.5 0 1.5(.5)5
global use_cf_estname=1
global scale_adjirelto=1
global cfonly=0
global output_ver=3

global interval=5
global shiftlens 7 8 9 10 12
global nshifttypes_7 9/11
global nshifttypes_8 12/13
global nshifttypes_9 14/35
global nshifttypes_10 1/6
global nshifttypes_12 7/8
global discharge_estimates clogit
global arrival_estimates ologit
global xbd_estimates xbd_lntdc
global keepirelto_range 1 60 
global simirelto_range -60 60
global estfile ${Data}/estimates_dp

global b lntdc_ptadj
if "${keepirelto_range}"=="1 0" {
	global b_supp b_supp
}

*** Programs *****************************************************************************
capture program drop reg_storeb 
program reg_storeb
syntax , b(string) [const_b b_supp]
	preserve
	if "`b_supp'"!="" {
		append using "${Data}/stored_dp_data_supp"
		local _supp _supp
	}
	reg sim_`b' ib(last).irelto from?
	mat b=e(b)
	foreach irelto of numlist 1/7 {
		scalar b_`irelto'`_supp'=b[1,`irelto']
	}
	if "`const_b'"=="const_b" {
		if "${keepirelto_range}"=="1 0" {
			if $cfonly {
				if "`b_supp'"=="" gen const_`b'=sim_`b'- ///
					(from1*-.2044271+from2*.4464034+from3*-.9423602)
				else gen const_`b'=sim_`b'- ///
					(from1*-.2040081+from2*.4293106+from3*-.9470909)
			}
			else {
				if "`b_supp'"=="" gen const_`b'=sim_`b'- ///
					(from1*-.17730976+from2*.36121971+from3*-.72772702)
				else gen const_`b'=sim_`b'- ///
					(from1*-.17738804+from2*.34697671+from3*-.74264642)
			}
		}
		else if "${keepirelto_range}"=="1 60" {
			if $cfonly gen const_`b'=sim_`b'- ///
				(from1*-.1750032+from2*.3452114+from3*-.6819641)
			else gen const_`b'=sim_`b'- ///
				(from1*-.17253403+from2*.32072827+from3*-.6144076)
		}
		// Coefficients obtained from cf_extend==0
		reg const_`b' ib(last).irelto
		mat b=e(b)
		foreach irelto of numlist 1/7 {
			scalar const_b_`irelto'`_supp'=b[1,`irelto']
		}
	}	
end

capture program drop sum_save
program sum_save
syntax , cf_extend(real) rep(integer) file(string) b(string) [savemacro(string) ///
	varlist(string) const_b b_supp]
qui {
	// Regression
	preserve
	capture drop irelto
	gen irelto=min(7,ceil(irelto_base*${interval}/60))
	gen irelfrom=cond(irelfrom_base<=0,.,min(13,ceil(irelfrom_base*${interval}/60)))
	sum_sim_data, nocollapse types(sim)
	mkspline from=irelfrom, knots(1 3 5 7) cubic
	reg_storeb, b(`b') `const_b'
	reg_storeb, b(`b') `const_b' `b_supp'
		
	// Order/summary statistics 
	foreach irelto of numlist 1/7 {
		count if irelto==`irelto'
		local count`irelto'=r(N)
		if "`varlist'"!="" foreach var in `varlist' {
			sum sim_`var' if irelto==`irelto'
			local mean_sim_`var'_`irelto'=r(mean)
		}
	}
	sort nshiftid sim_dcirelto
	// Order patient from last to first discharged
	foreach ptorder of numlist 1/4 {
		by nshiftid: gen tr_eos_dc`ptorder'= ///
			cond(sim_dcirelto[`ptorder']>=0,0,-sim_dcirelto[`ptorder'])*${interval}/60
	}
		
	keep nshiftid *eos_dc?
	duplicates drop
	foreach ptorder of numlist 1/4 {
		qui	sum tr_eos_dc`ptorder'
		local mean_tr_eos_dc`ptorder'=r(mean)
	}	

	// Save to .dta file
	clear
	set obs 1
	gen byte rep=`rep'
	gen cf_extend=`cf_extend'
	if "`savemacro'"!="" {
		foreach macro in `savemacro' {
			if "${`macro'}"=="" disp as error "Warning: macro `macro' is missing"
			if wordcount("${`macro'}")==1 gen `macro'=${`macro'}
			else gen `macro'="${`macro'}"
		}
	}
	gen double datetime=clock("`c(current_date)' `c(current_time)'","DMYhms")
	format datetime %tc
	foreach coeff in b `const_b' {
		foreach irelto of numlist 1/7 {
			gen `coeff'`irelto'=scalar(`coeff'_`irelto')
			if "`b_supp'"!="" gen `coeff'`irelto'_supp=scalar(`coeff'_`irelto'_supp)
		}
	}
	foreach irelto of numlist 1/7 {
		gen count`irelto'=`count`irelto''
	}
	if "`varlist'"!="" foreach var in `varlist' {
		foreach irelto of numlist 1/7 {	
			gen `var'_`irelto'=`mean_sim_`var'_`irelto''
		}
	}
	foreach ptorder of numlist 1/4 {
		gen tr_eos_dc`ptorder'=`mean_tr_eos_dc`ptorder''
	}	
	restore, not
	compress
	capture confirm file `file'
	if _rc==0 append using `file'
	save "`file'", replace 
}
end

*** Run simulations **********************************************************************
set more off
foreach rep of numlist $rep_list {
disp as text _n ///
	"*** REPETITION `rep' *****************************************************"
	
foreach cf_extend of numlist $cf_extend_list {
noi disp as text _n "rep = `rep' ; cf_extend = `cf_extend'" _n

tempfile simobs
local appendsim=0
local cfparams cf_from(`=abs(`cf_extend')') cf_extend(`cf_extend')
qui foreach shiftlen of numlist $shiftlens {
	set_global_knots, shiftlen(`shiftlen')
	global estname dp_estimates_l`shiftlen'_a
	if `cf_extend'!=0&${use_cf_estname} {
		if sign(`cf_extend')==1 local cfname _cf+`cf_extend'
		else if sign(`cf_extend')==-1 local cfname _cf`cf_extend'
		global cf_estname ${estname}`cfname'
	}
	else global cf_estname ${estname}	
		
	local numlist ${nshifttypes_`shiftlen'}
	foreach nshifttype of numlist `numlist' {
		noi disp as text "nshifttype == `nshifttype': $S_DATE $S_TIME"
		use "${Data}/stored_dp_data.dta", clear
		local opt interval(${interval})
		local opt `opt' template(${Data}/sim_int5_d_template)
		local opt `opt' arrival_estimates(${arrival_estimates}_len`shiftlen'_`nshifttype')
		local opt `opt' xb_lntdc_estimates(${xbd_estimates}_len`shiftlen')
		local opt `opt' prep_dp_data_opts( if(shiftlen==`shiftlen') ///
			template(${Data}/sim_int5_d_template) nshifttypes(`nshifttype') ///
			arrival_estimates(ologit_len`shiftlen') ///
			xbd_estimates(xbd_lntdc_len`shiftlen') ///
			incensus)
		local opt `opt' ext_estimates_opts(estname(${cf_estname}) estfile(${estfile}))
		local opt `opt' simirelto_range(${simirelto_range}) 
		local opt `opt' dpirelto_range(${keepirelto_range})
		local opt `opt' discharge_estimates(${discharge_estimates}_len`shiftlen') 
		local opt `opt' noseed `cfparams' scale_adjirelto(${scale_adjirelto})
		if ${cfonly} local opt `opt' cfonly
		run_sim if shiftlen==`shiftlen'&nshifttype==`nshifttype', `opt'
		if `appendsim' append using `simobs'
		save `simobs', replace
		local appendsim=1
	}
}

sum_save, rep(`rep') cf_extend(`cf_extend') b(${b}) ///
	savemacro(use_cf_estname scale_adjirelto cfonly keepirelto_range) ///
	file(${Data}/cf_sims_data.dta) ///
	varlist(t_ed_dcord diff_lntdc lntdc_ptadj avgcensus) ///
	const_b ${b_supp}
}
}
