/*****************************************************************************************
AUTHOR: David Chan
PURPOSE: Generate stored dataset for dynamic programming
NOTES: 
- sample==0 includes patients recorded as being in wrong pod given gen_reltimeext.ado; 
  as well as shifts that may be non-standard, ending at times later than usual shift times;
  or patients who stayed too long. These patients are still counted in the gen_census 
  command but are not included in the logit estimation above. The logit estimation 
  sample should be considered sample==1. 
- Some patients are excluded from both sample==0 and sample==1 by discretize_dta -trim- 
  option.
*****************************************************************************************/

global interval=5

*** Program to shift multiple discharges to adjacent times *******************************
capture program drop shift_mult_obs
program shift_mult_obs
syntax , torel(string) [interval(integer 60)]
qui {
	capture gen orig_periods=periods
	capture gen orig_dcord_t=dcord_t
	set seed 0
	capture gen rand=runiform()
	preserve
	
	tempfile nshiftids
	keep nshiftid
	duplicates drop
	save `nshiftids', replace
	
	schedule_in_intervals, interval(`interval') prehrs(4)
	gen double ed_tc=at_tc+msofminutes(`interval'/2)
	keep npid shiftbeg_tc shiftend_tc ed_tc nshiftid
	discretize_dta, interval(`interval')
	keep if irelto_base<=12*60/`interval' // Consistent with sample definition
	drop if dofc(shiftbeg_tc)>19358 
	drop irelto_base irelfrom_base
	rename ed_t dcord_t
	tempfile template
	merge m:1 nshiftid using `nshiftids', keep(match) nogen
	save `template', replace
	restore
	
	foreach relnum in `torel' {
		preserve
		tempfile select
		merge m:1 nshiftid npid dcord_t using `template', keep(using) nogen
		replace dcord_t=dcord_t-`relnum'
		gen byte toshift=1
		keep dcord_t toshift nshiftid npid
		save `select', replace
		restore
		merge m:1 nshiftid npid dcord_t using `select', keep(master match) nogen
		sort nshiftid npid dcord_t rand
		by nshiftid npid dcord_t: replace periods=periods+`relnum' ///
			if toshift==1&_N>1&_n==1&periods+`relnum'>0
		replace dcord_t=ed_t-1+periods
		drop toshift
	}
	drop rand
}
end


*** Generate data ************************************************************************
gen_working_data, poe(sum)
gen_var, visittime elix control outcomes dem_vit
gen_reltimeext, by(npid pod) posthrs(6) keepmaster

discretize_dta, interval(${interval}) trim 
gen byte sample=periods<=12*60/${interval}&periods>0&irelto_base<=12*60/${interval}& ///
	irelto_base!=.&shiftlen!=.	
	// See notes above
qui areg lntdc i.visit_dow i.nvisit_ym age age2 male female35 de? v? v?? p? ///
	dr? dl? triage_ed if sample, absorb(npid)
	predict xbd_lntdc, xbd	
	
// Other variables to be used later as summary statistics
gen_census, of(npid) at_from(ed) at_to(dcord) match interval(15) gens(avg) exact
	// Uses 15-minute intervals but doesn't make substantive difference
gen lntdc_ptadj=ln(t_ed_dcord/avgcensus)
gen hr_irelto=ceil(hours(shiftend_tc-ed_tc))
replace hr_irelto=7 if hr_irelto>7&hr_irelto!=.
gen hr_irelfrom=ceil(cond(ed_tc>shiftbeg_tc,hours(ed_tc-shiftbeg_tc),.))
replace hr_irelfrom=13 if hr_irelfrom>13&hr_irelfrom!=.
keep xbd_lntdc sb irelto_base periods ed_t dcord_t ed_hh shiftlen npid visit_td ///
	irelfrom_base lntdc sample t_ed_dcord ed_tc dcord_tc nvisit_ym visit_dow ///
	postovl nshiftid hr_irelto hr_irelfrom lntdc_ptadj avgcensus nshifttype

// Shift multiple discharges to adjacent times (see program above)
tempfile outsample master
save `master', replace
drop if sample
save `outsample', replace
use `master', clear
keep if sample
shift_mult_obs, torel(-1 1 -2) interval(${interval})
append using `outsample'

save "${Data}/stored_dp_data.dta", replace

*** Generate and store supplementary data with unknown shifttype *************************
// Identical data generation as in Table A-7.3
gen_working_data, poe(sum)
gen_var, visittime elix control outcomes team dem_vit
gen_reltimeext, by(pod npid) posthrs(6) keepmaster

discretize_dta, interval(15) trim
gen_census, of(npid) at_from(ed) at_to(dcord) match interval(15) gens(avg)
gen lntdc_ptadj=ln(t_ed_dcord/avgcensus)
gen irelto=ceil(hours(shiftend_tc-ed_tc))
replace irelto=7 if irelto>7&irelto!=.
gen irelfrom=ceil(cond(ed_tc>shiftbeg_tc,hours(ed_tc-shiftbeg_tc),.))
replace irelfrom=13 if irelfrom>13&irelfrom!=.
mkspline from=irelfrom, knots(1 3 5 7) cubic

keep if nshifttype==.
keep lntdc_ptadj irelto from? sb nshifttype
rename lntdc_ptadj sim_lntdc_ptadj
save "${Data}/stored_dp_data_supp.dta", replace
