/*****************************************************************************************
AUTHOR: David Chan
PURPOSE: Create binned scatter plot
OPTIONS:
*****************************************************************************************/

capture program drop binned_scatter 
program binned_scatter
syntax [if] [aweight/], yvar(string) xvar(string) [bins(integer 20) ///
	ylabel(string) xlabel(string) graphopts(string) msymbol(string)]
qui {
	set scheme s2mono
	preserve
*** Check options for consistency ********************************************************
	if "`if'"!="" keep `if'
	local mcolor black
	if "`weight'"!="" local weightexp [`weight'=`exp'] 
	if !regexm("`graph_opts'","legend") local graph_opts `graph_opts' legend(off)

	local num_yvars=wordcount("`yvar'") 
	if `num_yvars'==1 {
		disp as error "Error: must specify more than one yvar"
		exit
	}
	local objs yvar msymbol
	foreach obj in `objs' {
		if "`obj'"!="yvar"&wordcount("`msymbol'")!=`num_yvars' {
			disp as error "Error: -`obj'- objects don't match -yvar- objects"
			exit
		}
		tokenize "``obj''"
		foreach num of numlist 1/`num_yvars' {
			if "`obj'"=="yvar" local `obj'`num' ``num''
			else local `obj'_`yvar`num'' ``num''
		}
	}
	local numtype %9.3g

*** Begin analysis ***********************************************************************	
	keep `xvar' `yvar' `exp'
	
	tempfile temp master
	save `master'
	local yvarnum=1
	foreach var in `yvar' {
		use `master', clear
		drop if `var'==.
		
		// Regression fit
		reg `var' `xvar' `weightexp'
		if "`dispN'"=="" local dispN `=string(`=e(N)',"%15.0fc")'
		scalar _b_x=_b[`xvar']
		scalar _se_x=_se[`xvar']
		predict `var'_xxb, xb
	
		// Create bins
		local inc=100/`bins'	
		capture drop pc_x
		gen pc_x=.
		_pctile `xvar' if `var'!=. `weightexp', p(`inc'(`inc')`=100-`inc'')
		foreach num of numlist 1/`=`bins'-1' {
			replace pc_x=`num' if `xvar'<=r(r`num')& pc_x==.
		}
		sum `xvar' if `var'!=.
		replace pc_x=100 if `xvar'<=r(max)&pc_x==.

		// Means within bin
		capture drop totwt
		if "`exp'"!="" {
			by pc_x, sort: egen totwt=total(`exp')
			local wtexp *`exp'/totwt
		}
		else {
			by pc_x, sort: egen totwt=count(_n)
			local wtexp /totwt
		}
		by pc_x, sort: egen `xvar'`yvarnum'_mn=total(`xvar' `wtexp')
		by pc_x: egen `var'_resid_xmn=total(`var' `wtexp')
		by pc_x: egen `var'_xb_xmn=total(`var'_xxb `wtexp')
		local keepvars_`var' `keepvars_`var'' `xvar'`yvarnum'_mn `var'_resid_xmn `var'_xb_xmn
		
		// Macros for plotting
		local plotscatters `plotscatters' ///
			(scatter `var'_resid_xmn `xvar'`yvarnum'_mn, mcolor(`mcolor') msymbol(`msymbol_`var''))
		
		// Merge
		keep `keepvars_`var''
		duplicates drop
		sort `xvar'`yvarnum'_mn
		gen byte order=_n		
		if `yvarnum'>1 merge 1:1 order using `temp', nogen
		save `temp', replace
		local yvarnum=`yvarnum'+1
	}
		
	// Plot 
	local _b `=string(_b_x,"`numtype'")'
	local _se `=string(_se_x,"`numtype'")'
	foreach var in `yvar' {
		sum `var'_resid_xmn
		if "`min'"!="" local min=min(r(min),`min')
		else local min=r(min)
		if "`max'"!="" local max=max(r(max),`max')
		else local max=r(max)
		local range=`max'-`min'
	}	
	
	if "`ylabel'"!="" local ytitle ytitle(`ylabel')
	if "`xlabel'"!="" local xtitle xtitle(`xlabel')
	twoway `plotscatters', ///
		`graphopts' ytitle(`ylabel') xtitle(`xlabel') ylabel(, nogrid) ///
		yscale(range(`=`max'-(`range'*1.15)' `max')) ///
		graphregion(color(white))
		
	restore
	capture set scheme sol
}
end
