/*****************************************************************************************
AUTHOR: David Chan
PURPOSE: Save coefficients currently stored as vectors into a .dta file; retrieve 
coefficients into a single vector from the .dta file
OPTIONS:
- append: Save estimates from text file (useresults)
- estname: Name of estimate in estfile
- estfile: Name of file that estimate is stored in
- predict: Use saved coefficients to make linear prediction
- useresults: Save estimates from text file (produced by MATLAB). Must be specified by 
  append.
******************************************************************************************/

capture mata: mata drop matapredict()
mata:
void matapredict(string scalar varnames_name, 
				 string scalar betas_name, 
				 string scalar deltavec_name,
				 string scalar predict)
{
	real colvector betas
	real colvector deltavec
	string rowvector varnames
	real colvector xb
	real matrix xvars
	
	varnames=tokens(st_local(varnames_name))
	st_view(xvars=.,.,varnames)
	deltavec=st_matrix(deltavec_name)
	betas=st_matrix(betas_name)
	xb=xvars*(deltavec:*betas)
	st_store(.,predict,xb)
}
end		

capture program drop ext_estimates_make_varlist 
program ext_estimates_make_varlist 
syntax , estname(string) estfile(string) [estname_short(string)]
	if "`estname_short'"=="" {
		noi disp as error "Note: global estname_short not specified"
	}
	preserve

	foreach type in u_ w_ Ew_ num h_ Eh_ {
	if "`estname_short'"!="" local `type' `estname_short'_`type'
	else local `type' `type'
	}
	foreach type in w u h {
		local num`type'vars=0
	}
	foreach type in u_ w_ Ew_ {
		global ``type''vars
		global ``type''varlabs
		capture drop `type'var*
	}

	use "`estfile'", clear
	keep if estname=="`estname'":estname
	if _N==0 {
		noi disp as error "Error: `estname' not found in `estfile'"
		exit
	}
	else {
		tempfile select_estimates
		decode nvar, gen(var)
		decode nmatlabvar, gen(matlabvar)
		save `select_estimates', replace
		foreach nametype in u_ Ew_ {
			use `select_estimates', clear
			keep if regexm(matlabvar,"`nametype'")
			if "`nametype'"=="u_" local type u
			else local type w
			scalar `num'`type'vars=_N
			foreach n of numlist 1/`=_N' {
				noi disp as text var[`n'] _col(40) " = " as result coeff[`n']
				global ``nametype''vars ${``nametype''vars} `=matlabvar[`n']'
				global ``nametype''varlabs ${``nametype''varlabs} ///
					`=var[`n']'
			}
		}
		restore
		
		foreach type in u Ew {
			if "`type'"=="Ew" local scalartype w
			else local scalartype u
			foreach i of numlist 1/`=scalar(`num'`scalartype'vars)' {
				local var `=word("${``type'_'vars}",`i')'
				local exp `=word("${``type'_'varlabs}",`i')'
				local exp `=subinstr("`exp'","#","*",.)'
				local exp `=subinstr("`exp'","c.","",.)'
				local label `=word("${``type'_'varlabs}",`i')'
				gen `var'=`exp'
				label var `var' "`label'"
			}
		}
	}
end

capture program drop useresults
program useresults, rclass
syntax , file(string) varver(string) [delta(real 1)]
{
	local error = 0
	insheet using "`file'", tab clear
	capture drop v2
	rename v1 coeff
	tempfile results
	if `=round(coeff[1]*100)'>=0&`=round(coeff[1]*100)'<=100 {
		if `=round(`delta'*100)'!=`=round(coeff[1]*100)' {
			disp as error "Warning: asserted delta does not match in results file"
			local error = 1
			exit
			/*
			disp as error "Note: changing delta to `=coeff[1]'"
			global delta=`=round(coeff[1]*100)/100'
			*/
		}
		drop if _n==1
	}
	else {
		disp as error "Error: delta in results file not recognized as valid"
		local error = 1
		exit
	}
	save `results', replace

	proc_dp_varlist, label estname(`varver')
	merge 1:1 _n using `results', assert(match) nogen
	save `results', replace
	clear
	set obs 1
	gen coeff=`delta'
	gen var=""
	gen matlabvar="delta"
	append using `results'
	return scalar error = `error'
}
end

capture program drop ext_estimates
program ext_estimates
syntax , estfile(string) [estname(string) predict(string) replace append ///
	nodisp delta(real -1) useresults(string)]
qui {
	if "`predict'"!="" local retrieve retrieve
	local check `retrieve' `append'
	if wordcount("`check'")==0 {
		disp as error "Error: Mode not specified"
		exit
	}
	if "`disp'"=="nodisp"&"`retrieve'"=="" {
		disp as error "Error: -nodisp- not valid without -predict-"
	}
	if "`retrieve'"!="" {
		local betas_name betas
		local varnames_name varnames
	}
	if "`useresults'"!=""&"`append'"=="" {
		disp as error "Error: cannot specify -useresults- without -append-"
	}
	if "`estname'"=="" {
		capture confirm var estname
		if _rc!=0 {
			capture confirm var strname
			if _rc!=0 {
				disp as error "Error: estname not found and not specified"
				exit
			}
		}
	}
	preserve
	
	if "`append'"!="" {
		capture use "`estfile'", clear
		if _rc==0 {
			count if estname=="`estname'":estname
			if r(N)>0&"`replace'"=="" {
				disp as error "Error: `estname' already exists in `estfile'"
				exit
			}
		}

		set more off
		tempfile coeffs
		local nmatlabvar nmatlabvar
		if "`useresults'"!="" {
			useresults, `useresults'
			if r(error) {
				disp as error "Error: -useresults- returned with error-"
				exit
			}
		}
		else {
			restore
			preserve
		}
		capture decode nvar, gen(var)
		if _rc==0 drop nvar
		capture decode estname, gen(strname)
		if _rc==0 drop estname
		
		capture gen strname="`estname'"
		capture gen double savedat_tc= ///
			clock("`c(current_date)' `c(current_time)'","DMYhms")
		save `coeffs', replace

		capture use "`estfile'", clear
		if _rc==0 {
			if "`replace'"!="" {
				capture drop if estname=="`estname'":estname
			}
			decode nvar, gen(var)
			decode estname, gen(strname)
			capture decode nmatlabvar, gen(matlabvar)
			drop nvar estname
			capture drop nmatlabvar
			append using `coeffs'
		}

		encode var, gen(nvar)
		encode strname, gen(estname)
		capture encode matlabvar, gen(nmatlabvar)
		drop var strname
		capture drop matlabvar
		order estname nvar `nmatlabvar' coeff savedat_tc
		format *_tc %tc
		save `estfile', replace
		restore
	}
		
	// Retrieve coefficients from `estfile'
	if "`retrieve'"!="" {
		use "`estfile'", clear
		keep if estname=="`estname'":estname
		if _N==0 {
			noi disp as error "Error: `estname' not found in `estfile'"
			exit
		}
		decode nmatlabvar, gen(matlabvar)
		if `delta'==-1 {
			count if matlabvar=="delta"
			if r(N)!=1|matlabvar[1]!="delta" {
				noi disp as error "Error: delta placed unexpectedly or not found"
				exit
			}
			local delta=`=round(coeff[1]*100)/100'
		}
		assert `delta'>=0&`delta'<=1
		if "`disp'"=="" noi disp as text "delta" _col(40) " = " as result `delta'		

		decode nvar, gen(var)
		foreach n of numlist 1/`=_N' {
			if matlabvar[`n']!="delta" {
				if "`disp'"=="" noi disp as text var[`n'] _col(40) " = " ///
					as result coeff[`n']
				local varname `=var[`n']'
				local `varnames_name' ``varnames_name'' `varname'
				if "`predict'"!=""&regexm("`varname'","eos") {
					if regexm("`varname'","#")|!regexm("`varname'","EW_") {
						disp as error "Error: invalid eos variable"
						exit
					}
					local eosnames `eosnames' `=regexr("`varname'","eos_","")'
				}
			}
		}
		if "`predict'"!="" {
			drop if matlabvar=="delta" 
			mkmat coeff, matrix(`betas_name')
			gen deltavar=cond(regexm(matlabvar,"Ew_"),`delta',1)
			mkmat deltavar, matrix(deltavec)
		}
		restore
		if "`predict'"!="" {
			capture drop `predict'
			gen `predict'=.
			if "`eosnames'"!="" {
				foreach var of varlist `eosnames' {
					gen `=regexr("`var'","EW_","EW_eos_")'=cond(irelto==1,`var',0)
					gen `=regexr("`var'","EW_","W_eos_")'=0
				}
				foreach var of varlist EW_* {
					if !regexm("`var'","eos") replace `var'=0 if irelto==1
				}
			}		
			mata: matapredict("`varnames_name'","`betas_name'", ///
				"deltavec","`predict'")
		}
	}	
}
end

