/*****************************************************************************************
AUTHOR: David Chan
PURPOSE: Generate patient census for physician, pod, or waiting room at given time 
OPTION NOTES:
- `of': identity to calculate census for (e.g., npid, pod, waiting room)
- `at_from' and `at_to': censuses will be averaged across times from `at_from' to `at_to'
- `gens': attaches prefixes in `gens' to each one of `matchofs', respectively
- `matchlag': matches lagged census by `matchlag' intervals (see `interval'); recommended
  if estimating policy function of assignment based on censuses (see 
  test_dyn_ovl_shift.do, similar to est_alpha_policy.do)
- `from' and `to': Assume patients are on censuses between these times
- `interval': interval spacing in minutes between census observations
- `_t': time integers of the form *_t, based on `begtime' clock("1jun2005 00:00","DMYhm"), 
  exist already
- `exact': adjust censuses for fact that they are counted in discrete time
******************************************************************************************/

capture program drop gen_census
program gen_census
syntax , of(string) [to(string) gens(string) match interval(integer 60) ///
	matchlag(integer 0) at_from(string) at_to(string) _t exact]

if "`of'"=="waiting" {
	local of 
	local ofwaiting waiting room
}
if "`_t'"=="_t" local _tc _t
else local _tc _tc
// Default settings
if "`of'"=="waiting" {
	local from triage
	if "`to'"=="" local to ed
}
else {
	local from ed
	if "`to'"=="" local to dcord
}
if "`at_from'"==""|"`at_to'"=="" {
	local at ed
	local atline `at'_tc
}
else local atline (`at_from'`_tc' to `at_to'`_tc')


disp as text "Generating patient census for `of'`ofwaiting' at `atline' ..."
disp as text "  Note: census defined from `from'`_tc' to `to'`_tc'"
disp as text "  Note: census defined for each interval of `interval' minutes"
		
local begtime clock("1jun2005 00:00","DMYhm")

preserve
if "`match'"!="" {
	local matchofs `of'
	disp as text "  Note: matching to `matchofs', at `atline', lagged by `matchlag' interval(s)"
	if "`match'"!=""&"`gens'"=="" {
		disp as text "  Note: assuming prefix `of' for matched census"
		local gens `of'
	}
	if wordcount("`matchofs'")!=wordcount("`gens'") {
		disp as error "  Error: number of censuses and match destinations different"
		exit
	}
	else local nummatches=wordcount("`matchofs'")
	// Tag master data with at_t for future merging
	qui foreach var in at_t at_from_t at_to_t {
		capture drop `var'
	}
	qui {
		if "`at'"!="" {
			if "`_t'"=="" gen at_t=ceil(minutes(`at'_tc-`begtime')/`interval')
			else gen at_t=`at'_t
			local mergefile master
		}
		else {
			if "`_t'"=="" {
				gen at_from_t=ceil(minutes(`at_from'_tc-`begtime')/`interval')
				gen at_to_t=ceil(minutes(`at_to'_tc-`begtime')/`interval')
			}
			else {
				gen at_from_t=`at_from'_t
				gen at_to_t=`at_to'_t
			}
			local mergefile master_narrow
		}
	}
}
else if `matchlag'!=0 {
	disp as error "  Error: cannot specify -matchlag-"
	exit 
}
	
qui {
*** Main algorithm ***********************************************************************
	capture confirm var pseudosb
	if _rc==0 {
		by pseudosb, sort: assert _N==1
		local sb pseudosb
	}
	else {
		capture by sb, sort: assert _N==1
		if _rc!=0 {
			rename sb sb_orig
			gen pseudosb=_n
			local sb pseudosb
		}
		else local sb sb
	}
	
	if "`match'"!=""|"`matchofs'"!="" {
		tempfile master master_narrow census
		save `master', replace
	}

	if "`use'"!="" {
		use `use', clear
		save `census', replace
	}
	else {
		foreach var in from_t to_t {
			capture drop `var'
		}
		if "`_t'"=="" {
			gen to_t=ceil(minutes(`to'_tc-`begtime')/`interval')
			gen from_t=ceil(minutes(`from'_tc-`begtime')/`interval')
			if "`exact'"=="exact" {
				gen to_t_wt=minutes(`to'_tc-`begtime')/`interval' - ///
					floor(minutes(`to'_tc-`begtime')/`interval')
				gen from_t_wt=ceil(minutes(`from'_tc-`begtime')/`interval')- ///
					minutes(`from'_tc-`begtime')/`interval'
				local wts to_t_wt from_t_wt
			}
		}
		else {
			gen to_t=`to'_t
			gen from_t=`from'_t
		}
		assert to_t!=.
		if "`ofwaiting'"!="" {
			capture gen pod="WAITING"
			replace pod="WAITING"
			by from_t, sort: egen volume=count(_n)
			local ofwaiting pod
			local varlist volume
		}
		keep to_t from_t `ofwaiting' `of' `sb' `varlist' `wts'
		foreach var in at_t nt {
			capture drop `var'
		}
		gen nt=to_t-from_t+1
		expand nt
		by `sb', sort: gen at_t=from_t+_n-1
		if "`exact'"=="exact" {
			if "`_t'"=="" by `sb': gen wt=cond(_n==1&_n==_N,from_t_wt+to_t_wt-1, ///
				cond(_n==1,from_t_wt,cond(_n==_N,to_t_wt,1)))
			else by `sb': gen wt=cond(_n==1&_N==1,.3,cond(_n==1|_n==_N,.5,1))
			local exact_weighted wt*
		}
		
		// Key part that generates censuses
		by at_t `of', sort: egen census=total(`exact_weighted'1)
		
		if "`varlist'"!="" ///
			foreach var of varlist `varlist' `rel_varlist' {
			by at_t `of': egen avg_`var'=mean(`var')
			local avg_varlist `avg_varlist' avg_`var'
		}				
		gen double at_tc=clock("1jun2005 00:00","DMYhm")+ ///
			msofminutes(`interval'*at_t)
		format at_tc %tc
		keep at_t at_tc `of' `ofwaiting' census `avg_varlist'
		duplicates drop
		drop if at_t==.
		drop if missing(`of')
	}
	
	if "`matchofs'"!="" {
		replace at_t=at_t+`matchlag'
		save `census', replace
		if "`at_from'"!=""&"`at_to'"!="" {
			use `master', clear
			assert at_to_t!=.
			gen nt=at_to_t-at_from_t+1
			keep `sb' `matchofs' at_from_t at_to_t nt
			expand nt
			by `sb', sort: gen at_t=at_from_t+_n-1
			drop at_from_t nt
			save `master_narrow', replace
		}

		foreach num of numlist 1/`nummatches' {
			use `census', clear
			tokenize `matchofs'
			local matchof ``num''
			rename `of' `matchof'
			tokenize `gens'
			rename census ``num''census
			merge 1:m at_t `matchof' using ``mergefile'', keep(match using) nogen
			replace ``num''census=0 if !missing(`matchof')&``num''census==. 
			if "`at_from'"!=""&"`at_to'"!="" {
				rename ``num''census temp_``num''census
				by `sb', sort: egen ``num''census=mean(temp_``num''census)
				keep `sb' ``num''census
				duplicates drop
				merge 1:1 `sb' using `master', keep(match using) nogen
			}
			save `master', replace
		}
		capture drop at_t
	}
}

restore, not
end
