/*****************************************************************************************
AUTHOR: David Chan
PURPOSE: Shorter program to generate patient census for physician in settings where 
intervals are short and continuous, e.g., when estimating arrival probability 
function
NOTES:
- See gen_census.ado for much longer and more complicated program, designed to calculate
  censuses by expanding observations (unable to deal with dense settings)
******************************************************************************************/

capture program drop gen_census_dense
program gen_census_dense
syntax , [interval(integer 60) matchlag(integer 0)]
qui {
	noi disp as text "Generating census ..."
	**************************************************************************************
	// Settings
	local from ed
	local to dcord
	local at ed
	local npid npid
	local nshiftid nshiftid
	local begtime clock("1jun2005 00:00","DMYhm")
	noi disp as text ///
		"  Note: matching `npid' `nshiftid', at `at'_t in `interval'-minute intervals"
	noi disp as text "  Note: lagged by `matchlag' interval(s)"
	
	// Checks
	confirm variable `at'_t `npid' `nshiftid' sb 
	foreach var in `to' `from' {
		capture confirm variable `var'_tc
		if _rc!=0 capture confirm variable `var'_t 
		if _rc!=0 {
			noi disp as error "  Error: `var'_tc and `var'_t non-existent"
			exit
		}
	}
	
	// Check continuity of data, also require that at_t already discretized
	preserve
	tempfile sb_obs master ledger temp
	capture drop at_t
	gen at_t=`at'_t
	save `master', replace
	drop if `nshiftid'==.

	keep `npid' `nshiftid' at_t
	duplicates drop
	sort `npid' `nshiftid' at_t
	save `ledger', replace
	
	tempvar diff
	by `npid' `nshiftid': gen `diff'=cond(_n==1,1,at_t-at_t[_n-1])
	sum `diff'
	if r(min)!=1|r(max)!=1 {
		noi disp as error "  Error: `at'_t non-consecutive"
		exit
	}
	
	**************************************************************************************
	// Extract patient observations
	use `master', clear
	drop if `nshiftid'==.
	keep if sb!=.
	set varabbrev off
	foreach var in `to' `from' {
		capture d `var'_t
		if r(N)==. {
			local timevars `var'		
			count if `var'_tc==. 
			if r(N)!=0 {
				noi disp as error ///
					"  Warning: `var'_tc missing for some observations; dropping"
				drop if `var'_tc==.
			}
		}
		else {
			count if `var'_t==.
			if r(N)!=0 {
				noi disp as error ///
					"  Warning: `var'_t missing for some observations; dropping"
				drop if `var'_t==.
			}
		}
	}	
	if "`timevars'"!="" {
		keep `npid' `nshiftid' sb `to'_tc `from'_tc
		discretize_dta, interval(`interval') trim timevars(`timevars')
	}
	keep `npid' `nshiftid' sb `to'_t `from'_t
	count if `to'_t<`from'_t
	if r(N)!=0 {
		noi disp as error "  Warning: `from'_t > `to'_t for some observations; dropping"
		drop if `to'_t<`from'_t
	}
	save `sb_obs', replace
	
	keep if `to'_t==`from'_t
	rename `from'_t at_t
	by `npid' `nshiftid' at_t, sort: egen plus=total(.3)
	keep `npid' `nshiftid' at_t plus
	duplicates drop
	merge 1:1 `npid' `nshiftid' at_t using `ledger', nogen keep(match using)
	replace plus=0 if plus==.
	gen minus=plus
	save `ledger', replace
	
	use `sb_obs', replace
	keep if `to'_t>`from'_t
	expand 2
	save `sb_obs', replace
	by sb, sort: gen at_t=`from'_t+_n-1
	by `npid' `nshiftid' at_t, sort: egen inc=total(.5)
	keep `npid' `nshiftid' at_t inc
	duplicates drop
	merge 1:1 `npid' `nshiftid' at_t using `ledger', nogen keep(match using)
	replace plus=plus+inc if inc!=.
	drop inc
	save `ledger', replace
	use `sb_obs', clear
	by sb, sort: gen at_t=`to'_t+_n-1
	by `npid' `nshiftid' at_t, sort: egen inc=total(.5)
	keep `npid' `nshiftid' at_t inc
	duplicates drop
	merge 1:1 `npid' `nshiftid' at_t using `ledger', nogen keep(match using)
	replace minus=minus+inc if inc!=.
	sort `npid' `nshiftid' at_t 
	by `npid' `nshiftid': gen census=sum(plus)-sum(minus[_n-1])
	keep `npid' `nshiftid' at_t census
	replace at_t=at_t+`matchlag'
	
	merge 1:m `npid' `nshiftid' at_t using `master', nogen keep(match using)
	drop at_t
	set varabbrev on
}
restore, not
end
