/*****************************************************************************************
AUTHOR: David Chan
PURPOSE: Generate exact durations relative to shift times
NOTES:
OPTIONS:
- by: Can use by(npid), by(pod), or by(npid pod). If the last option, then it ensures that
  npid and pod both match, which rules out cases where patient may have originally been
  admitted to one pod then transferred to another with the later attending listed as the
  AOR.
- prehrs and posthrs: To be passed on to schedule_in_intervals.ado.
- keepmaster: Keep unmatched master data
- keepsched: Keep unmatched schedule data
- interval: Interval of time in minutes for schedule (default 60 minutes)
- retrievefrom: Specify name of previously saved schedule
- saveprep: Save schedule prepared in the process
*****************************************************************************************/

capture program drop gen_reltimeext
program define gen_reltimeext
syntax , by(string) [prehrs(integer 0) posthrs(integer 0) keepmaster keepsched ///
	interval(integer 60) at(string) retrievefrom(string) saveprep]

// Consistency checks
local shifttype_ext nshifttype preovl postovl shiftlen
if "`at'"=="" local at ed
if "`retrievefrom'"=="" local retrievefrom choice_hour_obs
noi disp as text "  Note: prepare schedule if `retrievefrom' invalid"
qui foreach var in shiftid shiftbeg_tc shiftend_tc shifttype nshifttype preovl ///
	postovl shiftlen podwn {
	capture d `var'
	if r(N)!=. local delete delete
	capture drop `var'
}
if "`delete'"=="delete" ///
	disp as error "  Note: gen_reltimeext already run, dropping prior values"
if `interval'!=60 {
	if ceil(60/`interval')!=60/`interval' {
		disp as error "  Error: 60 minutes must be multiple of interval"
		exit
	}
	if "`at'"=="" {
		disp as error "  Error: -at- root must be specified"
		exit
	}
	if "`keepsched'"=="" {
		disp as error "  Note: assuming -keepsched- option, given interval!=60"
		local keepsched keepsched 
	}
	qui {
		local begtime clock("1jun2005 00:00","DMYhm")
		capture drop `at'_t
		gen `at'_t=ceil(minutes(`at'_tc-`begtime')/`interval')
		local at_t `at'_t
		local time `at_t'
		// Just in case if interval in source data different than in schedule data
		rename `at'_tc old_`at'_tc 
	}
}	
else qui {
	local date `at'_td
	local hour `at'_hh
	local time `date' `hour'
	capture d `time'
	if r(N)==. gen_var, visittime
}
if "`by'"!="npid"&"`by'"!="pod"&"`by'"!="pod npid"&"`by'"!="npid pod" disp as error ///
	"  Warning: only by(npid), by(pod), or by(pod npid) debugged for -by-"

qui {
	confirm variable `by' `time'
	tempfile master
	save `master', replace
	
	if "`saveprep'"=="saveprep" {
		if "`retrievefrom'"!="" local save saveas(`retrievefrom')
		else local save save
	}
	noi schedule_in_intervals, prehrs(`prehrs') posthrs(`posthrs') ///
		save interval(`interval') retrievefrom(`retrievefrom')
}

if regexm("`by'","pod") {
	qui count if pod=="A"|pod=="B"|pod=="C"|pod=="ORANGE"|pod=="RED"|pod=="BLUE" ///
		|pod=="GREEN"|pod=="PURPLE"
	if r(N)>0 {
		local podwnroom podwnroom
		local room room 
		rename pod podwn
		local podwn podwn
		qui gen pod=cond(podwn=="ECNU","ECNU",room)		
		if `posthrs'==0&"`by'"=="pod" {
			disp as error ///
				"  Warning: recommend posthrs>0 if matching by pod after 5/2011"
		}
	} 
}

qui { // Main program
	if `interval'!=60 {
		rename at_t `at'_t
		rename at_tc `at'_tc
		local keep `at'_t `at'_tc
	} 
	else {
		rename date `date'
		rename hour `hour'	
	}
	keep npid *shiftid shiftbeg_tc shiftend_tc `by' `podwn' `room' ///
		`time' `keep' `shifttype_ext'
	
	if !regexm("`by'","npid") { // previously formulated as -if "`by'"!="npid"-
		sort `time' `by' shiftbeg_tc
		by `time' `by': gen j=_n
		rename npid npidsched
		reshape wide npidsched shiftid shiftbeg_tc shiftend_tc `shifttype_ext' `podwn', ///
			i(`time' `by') j(j)
	} 

	if "`keepmaster'"=="keepmaster" local merge1 using
	if "`keepsched'"=="keepsched" local merge2 master
	merge 1:m `time' `by' using `master', nogen keep(match `merge1' `merge2') 
	
	if !regexm("`by'","npid") { // Determine podwn and drop pods that don't match
		reshape long npidsched shiftid shiftbeg_tc shiftend_tc `shifttype_ext' `podwn', ///
			i(sb) j(j)
		gen aor=cond(npid==npidsched,1,0)
		by `time' podwn, sort: egen hasaor=max(aor)
		keep if hasaor
		drop hasaor aor
		drop if npidsched==.
		sort sb shiftbeg_tc
		by sb: replace j=_n
		reshape wide
	}
}
end
