/*****************************************************************************************
AUTHOR: David Chan
PURPOSE: Generate variables
NOTES: Central code to generate variables after creating datasets and before analysis
*****************************************************************************************/

capture program drop gen_var
program define gen_var
syntax , [outcomes control visittime elix team costs dem_vit room]

	// demographics and vital statistics
	if "`dem_vit'"=="dem_vit" {
		disp as text "  Demographics and vital statistics"
		qui {
			merge m:1 mrn using "${Data}/dem_vit_data", ///
				keep(match match_up match_con master) nogen update
			gen death_td=date(date_of_death,"MDY")
			format death_td %td
		}
	}
	
	// costs
	if "`costs'"=="costs" {
		disp as text "  Costs"
		qui {
			merge 1:1 sb mrn using "${Data}/cost_data", keep(match master) nogen
			gen lntotcost=ln(1+Total_Direct_Costs)
			drop *Cost*
		}
	}
		
	// outcomes
	if "`outcomes'"=="outcomes" {
		disp as text "  Outcomes"
		qui {
			gen lntdc=ln(t_ed_dcord+.01)
			gen byte mort30d=(death_td-visit_td)<=30
			sort mrn visit_td sb
			by mrn: gen dbwvis=visit_td[_n]-visit_td[_n-1]
			capture drop bounceback
			gen byte bounceback=dbwvis<=14&(destination=="C C  - LEFT WITHOUT"| ///
				destination=="H HOME - ROUTINE"|destination=="W WALKOUT - ELOPED"| ///
				destination=="E EDOBS")
			gen byte admit=destination=="A ADMIT"
			capture gen lntord=ln(t_ed_ord+.01)
			capture gen lnt1ord=ln(t_ed_npid1ord+.01)
			capture gen lntaorord=ln(t_ed_aorord+.01)
			capture gen ln1ordtdc=ln(t_ed_dcord-t_ed_npid1ord)
			capture gen lnlordtdc=ln(t_lastord_dcord)
		}
	}
		
	// Elixhauser indices
	if "`elix'"=="elix" {
		disp as text "  Elixhauser indices"
		qui {
			merge 1:1 sb using "${Data}/haselix_data", keep(master match)
			rename _merge _oldmerge
			merge 1:1 sb using "${Data}/noelix_data", keep(master match)
			foreach var of varlist v? v?? {
				replace `var'=0 if `var'==.&_merge==3
			}
			keep if _merge==3|_oldmerge==3
			drop _merge _oldmerge
		}
	}

	// descriptors of visit date
	if "`visittime'"=="visittime" {
		disp as text "  Visit-time dummies"
		qui {
			capture gen visit_m=month(visit_td)
			capture gen visit_dow=dow(visit_td)
			capture gen ed_hh=hh(ed_tc)
			capture gen ed_td=dofc(ed_tc)
			capture gen dcord_td=dofc(dcord_tc)
			capture gen dcord_hh=hh(dcord_tc)
			capture gen int visit_y=year(visit_td)
			capture gen visit_ym=string(visit_y)+"_"+string(visit_m,"%02.0f")
			capture gen nvisit_ym=(visit_y-2005)*12+visit_m-1
			format *_td %td
		}
	}
	
	// other variables
	if "`control'"=="control" {
		disp as text "  Other control variables"
		qui {
			capture gen triage_ed=hours(ed_tc-triage_tc)
			capture gen lntred=ln(triage_ed+.01)
			gen age2=age^2
			gen female35=age<35&!male
			tab esi, gen(de)
			gen tr_edXp1=triage_ed*p1

			// race (omitted variable other)
			gen byte dr1=inlist(race,"African American","BLACK OR AFRICAN AMERICAN", ///
				"Black-B","Black-BLACK")
			gen byte dr2=inlist(race,"American Indian White-AM IND/WHITE", ///
				"American Indian-AMER. INDIAN","Hispanic Black-HIS BLACK", ///
				"Hispanic White-HIS WHITE","Hispanic-H","Hispanic-HISPANIC", ///
				"Hispanic-HISPANIC OR LATINO","Hispanic-LATINO","Hispanic-SPANISH")
			gen byte dr3=inlist(race,"White-CAUCASIAN","White-W","White-WHITE")
			gen byte dr4=inlist(race,"Hawaiian-NAT. HAWAIIAN", ///
				"Hawaiian-NATIVE HAWAIIAN OR OTHER PACIF", ///
				"Hawaiian-NATIVE HAWAIIAN/OTHER PACIFIC","Oriental-ORIENTAL")
			replace dr4=1 if inlist(race,"Asian American Indian-ASIAN AMER IND", ///
				"Asian Black-BLACK/ASIAN","Asian Hawaiian-ASIA/NAT HAWAII", ///
				"Asian Pacific Islander-ASIAN PAC ISL","Asian White-ASIAN/WHITE", ///
				"Asian-A","Asian-ASIAN")
			
			// language (omitted variable other)
			gen byte dl1=inlist(language,"English-ENGLISH","English-ENGL")
			gen byte dl2=inlist(language,"Spanish-SPANISH","Spanish English-SPAN-ENGL", ///
				"Spanish-SPAN","Portuguese-PORTUGUESE","Portuguese-PORT")
		}
	}
				
	// team dummies
	if "`team'"=="team" {
		disp as text "  Team dummies"
		qui gen team=string(npid)+res_pa+nurse
	}
	
	// room identities
	if "`room'"=="room" {
		disp as text "  Room identities"
		qui {
			capture drop room
			capture drop nroom
			gen room="ALPHA" if pod=="ALPHA"|pod=="RED"|pod=="BLUE"|pod=="A"
			replace room="BRAVO" if pod=="BRAVO"|pod=="ECNU"|pod=="GREEN"| ///
				pod=="PURPLE"|pod=="B"
			replace room="CHARLIE" if pod=="CHARLIE"|pod=="ORANGE"|pod=="C"
			encode room, gen(nroom)
		}
	}
			
end
