/*****************************************************************************************
AUTHOR: David Chan
PURPOSE: Generate data for logit estimation of discharge hazard
NOTES: 
******************************************************************************************/

capture program drop make_logit_dta
program make_logit_dta
disp as text "Preparing data for discrete hazard logit ..."
syntax , [keepvars(string) interval(integer 60) visits1h lntdc_ptadj]
qui {	
	local begtime clock("1jun2005 00:00","DMYhm")
	count if !sample
	if r(N)==0 {
		disp as error "  Error: need sample=0 to be relevant"
		exit
	}
	local keepvars `keepvars' dcord_t nshifttype
	local nshifttype `nshifttype'
	
	local reqvars irelto_base nshiftid irelfrom_base sample xbd_lntdc `nshifttype' ///
		periods ed_t sb
	local optvars npid shiftlen visit_td ed_hh
	foreach var in `reqvars' `keepvars' {
		confirm var `var'
	}
	foreach var in `optvars' {
		capture confirm var `var'
		if _rc==0 local keepoptvars `keepoptvars' `var'
	}
	keep `keepvars' `reqvars' `keepoptvars'
	if "`visits1h'"=="visits1h" {
		tempfile data
		save `data', replace
	}

	tempfile outsample outsamplenums master
	preserve
	keep if !sample
	by nshiftid, sort: gen outid=_n
	save `outsample', replace
	by nshiftid: egen numout=count(_n)
	by nshiftid: keep if _n==1
	keep  nshiftid numout
	save `outsamplenums', replace
	restore
	drop if !sample
	expand periods
	
	by sb, sort: gen period=_n
	by sb: gen byte dc=_n==periods
	gen at_t=ed_t+period-1
	
	by nshiftid at_t, sort: egen census=count(_n)

	save `master', replace
	keep nshiftid at_t
	duplicates drop
	merge m:1 nshiftid using `outsamplenums', keep(match) nogen
	expand numout
	by nshiftid at_t, sort: gen outid=_n
	merge m:1 nshiftid outid using `outsample', keep(match) nogen
	by nshiftid at_t, sort: egen outcensus=total(at_t>=ed_t&at_t<=dcord_t)
	keep nshiftid at_t outcensus
	duplicates drop
	merge 1:m nshiftid at_t using `master', keep(using match) nogen
	replace census=census+outcensus if outcensus!=.
	drop outcensus
	
	if "`lntdc_ptadj'"=="lntdc_ptadj" {
		sort nshiftid sb at_t
		by nshiftid sb: gen u_tdc_ptadj=sum(1/census)
		gen u_lntdc_ptadj=ln(u_tdc_ptadj)
	}
	
	gen sim_lntdc=ln(cond(period==1,.5,period)/60*${interval})
	gen diff_lntdc=sim_lntdc-xbd_lntdc
	gen hr_ed_hh=hh(`begtime'+msofminutes(`interval'*at_t))
	gen irelto=(irelto_base-(period-1))  
	// Given parameters above, irelto>1-12*60/`interval'+1 and irelto<12*60/`interval'-1+1
	gen adjirelto=irelto+12*60/`interval'-1 // ensures no negative integers
	
	keep if sample
	drop sample
	set seed 0
	capture gen rand=runiform()
	sort nshiftid at_t dc rand
	by nshiftid at_t dc: replace dc=0 if _n>1 
	by nshiftid at_t: egen hasdc=max(dc)
	preserve
	tempfile at_t
	keep nshiftid at_t hasdc nshifttype
	duplicates drop
	foreach var in irelto_base adjirelto census diff_lntdc xbd_lntdc ///
		hr_ed_hh sb dc {
		gen `var'=0
	}
	save `at_t', replace
	restore
	append using `at_t'
	gen byte choice=dc
	gen byte nodc=sb==0
	replace choice=1 if !hasdc&sb==0
	egen groupid=group(nshiftid at_t)	
	
	preserve
	tempfile lastdc
	keep nshiftid at_t hasdc
	duplicates drop
	sort nshiftid at_t
	gen lastdc_t=.
	by nshiftid: replace lastdc_t=at_t[_n-1] if hasdc[_n-1]
	by nshiftid: replace lastdc_t=lastdc_t[_n-1] ///
		if lastdc_t==.&lastdc_t[_n-1]!=.
	save `lastdc', replace
	restore
	merge m:1 nshiftid at_t using `lastdc', assert(match) nogen
	capture replace lastdc_t=. if nodc
	
	if "`visits1h'"=="visits1h" {
		tempfile visits working
		save `working', replace
		keep nshiftid at_t
		duplicates drop
		expand 1+60/`interval'
		by nshiftid at_t, sort: gen ed_t=at_t-_n+1
		save `visits', replace
		use `data', clear
		by nshiftid ed_t, sort: egen visits=total(sample)
		by nshiftid ed_t: keep if _n==1
		keep nshiftid ed_t visits
		merge 1:m nshiftid ed_t using `visits', keep(using match) nogen
		replace visits=0 if visits==.
		sort nshiftid at_t ed_t
		by nshiftid at_t: gen visits1h_1=visits[1]
		by nshiftid at_t: egen visits1h=total(visits)
		keep nshiftid at_t visits1h visits1h_1
		by nshiftid at_t: keep if _n==1
		merge 1:m nshiftid at_t using `working', nogen
	}
}
end
