/*****************************************************************************************
AUTHOR: David Chan
PURPOSE: Flexible program to make splines to estimate models and simulate counterfactuals
******************************************************************************************/

capture program drop make_spline
program make_spline
syntax [if/], [prefix(string) var(string) estfile(string) knots(string) nknots(integer 0) ///
	clearall retrieve save]
qui {
	// Check for inconsistencies
	if `nknots'>0&"`knots'"!="" {
		noi disp as error "Error: cannot specify both -knots- and -nknots-"
		exit
	}
	if ("`clearall'"=="clearall"|"`retrieve'"=="retrieve"|"`save'"=="save") ///
		&"`estfile'"=="" {
		noi disp as error ///
			"Error: must specify estfile with -clearall-, -retrieve-, or -save-"
		exit
	}
	if "`clearall'"=="clearall" {
		if "`retrieve'"=="retrieve" {
			noi disp as error "Error: cannot specify both -clearall- and -retrieve-"
			exit
		}
	}
	if "`retrieve'"=="retrieve"&(`nknots'>0|"`knots'"!=""| ///
		"`prefix'"!=""|"`var'"!="") {
		noi disp as error ///
			"Error: cannot specify -retrieve- and other spline arguments"
		exit
	}
	if "`retrieve'"==""&"`clearall'"==""&`nknots'==0&"`knots'"=="" {
		noi disp as error "Error: must specify either -knots- or -nknots-"
		exit
	}
	if "`if'"!="" local shortif if `if'

	local splinefile ${Data}/knots_record
	tempfile temp_splinefile
	if "`clearall'"=="clearall"|"`save'"=="save" {
		preserve
		use "`splinefile'", clear
		notes renumber _dta
		local num=1
		local clearfile `estfile'
		while "`_dta[note`num']'"!="" {
			local string `_dta[note`num']'
			if word("`string'",1)=="`clearfile'" notes drop _dta in `num'
			local num=`num'+1
		}	
		notes renumber _dta
		save `temp_splinefile', replace
		restore
	}
	if "`retrieve'"=="retrieve" {
		preserve
		use "`splinefile'", clear
		local num=1
		local cmdnum=0
		while "`_dta[note`num']'"!="" {
			local string `_dta[note`num']'
			if (word("`string'",1)=="`estfile'") {
				local cmdnum=`cmdnum'+1
				local cmd`cmdnum' `=itrim("mkspline `=subinword("`string'",word("`string'",1),"",1)'")'
				local prefix`cmdnum' `=word(subinstr("`string'","="," ",.),2)'
				if regexm("`string'"," if ") {
					local ifpos=strrpos("`string'","if")
					local commapos=strrpos("`string'",",")
					local if`cmdnum' `=substr("`string'",`ifpos'+3,`commapos'-`ifpos'-3)'
				}
			}
			local num=`num'+1
		}
		local lastcmdnum=`cmdnum'
		restore
	}
	
	// Create splines retrieve previous parameters
	if "`retrieve'"=="retrieve" foreach cmdnum of numlist 1/`lastcmdnum' {
		capture drop `prefix`cmdnum''?
		`cmd`cmdnum''
		if "`if`cmdnum''"!="" {
			capture drop miss_`prefix`cmdnum''
			capture drop `prefix`cmdnum''0
			gen byte missing=1 if !(`if`cmdnum'')
			foreach var2 of varlist `prefix`cmdnum''? {
				replace `var2'=0 if missing==1
			}
			gen byte miss_`prefix`cmdnum''=cond(missing==1,1,0)
			gen byte `prefix`cmdnum''0=miss_`prefix`cmdnum''
			drop missing
		}
	}
	else if "`retrieve'"==""&!("`clearall'"!=""&`nknots'==0&"`knots'"=="") {
		if `nknots'>0 local cmd_nknots nknots(`nknots')
		else if "`knots'"!="" local cmd_knots knots(`knots')

		capture drop `prefix'?
		mkspline `prefix'=`var' `shortif', `cmd_knots' `cmd_nknots' cubic
		if "`if'"!="" {
			capture drop miss_`prefix'
			capture drop `prefix'0
			gen byte missing=1 if !(`if')
			foreach var2 of varlist `prefix'? {
				replace `var2'=0 if missing==1
			}
			gen byte miss_`prefix'=cond(missing==1,1,0)
			gen byte `prefix'0=miss_`prefix' // Alternate variable name
			drop missing
		}		

		if "`save'"=="save" {
			preserve
			if "`clearall'"=="clearall" use `temp_splinefile', clear
			else use "`splinefile'", clear
			if "`knots'"!="" ///
				note: `estfile' `prefix'=`var' , cubic knots(`knots')
			else if `nknots'>0 {
				mat rknots=r(knots)
				local estline `prefix'=`var' `shortif', cubic knots(
				foreach num of numlist 1/`nknots' {
					if `num'==1 local estline `estline'`=rknots[1,`num']'
					else local estline `estline' `=rknots[1,`num']'
				}
				local estline `estline') 
				note: `estfile' `estline'
			}
			save "`splinefile'", replace
			restore
		}
	}
}	
end
