/*****************************************************************************************
AUTHOR: David Chan
PURPOSE: Prepare data for dynamic programming (Barwick-Pathak implementation)
NOTES: 
- Creates (a) splines of characteristics entering utility flow, and (b) splines of 
  expectations of future state characteristics for value function 
******************************************************************************************/

*** Supporting programs ******************************************************************
capture program drop gen_spline_knot_macro
program gen_spline_knot_macro 
syntax , [var(string) nknots(integer 0) macroname(string) var_nknots(string) disp]
qui {
	local pctiles_3 10 50 90
	local pctiles_4 5 35 65 95
	local pctiles_5 5 27.5 50 72.5 95
	if "`var_nknots'"!=""&"`var'"==""&`nknots'==0 {
		tokenize "`var_nknots'", parse("=")
		local var `1'
		local nknots=`3'
	}
	if "`macroname'"=="" local macroname `var'_knots
	_pctile `var', percentiles(`pctiles_`nknots'')
	local `macroname'
	foreach num of numlist 1/`nknots' {
		local `macroname' ``macroname'' `=round(r(r`num')*10)/10'
	}
	global `macroname' ``macroname''
}
if "`disp'"=="disp" disp as text "  `macroname': " as result "${`macroname'}"
end

capture program drop gen_exp_trnorm3
// Generate cubed moment of truncated norm (truncation>0)
program gen_exp_trnorm3
syntax , mean(string) sd(string) gen(string)
qui {
	capture drop `gen'
	tempvar rho h m1 m2
	gen double `h'=-(`mean')/`sd'
	gen double `rho'= normalden(`h')/normal(-1*`h')
	gen double `m1'=`mean'+`rho'*`sd'
	gen double `m2'=`sd'^2*(1-`rho'*(`rho'-`h'))
	gen double `gen'=`sd'^3*`rho'*((`rho'-`h')^2+`rho'*(`rho'-`h')-1) 
	// Formulas for `m1', `m2', `gen' thus far from Horace (2015) Corollary 1
	replace `gen'=`gen'+`m1'^3+3*`m1'*`m2' 
		// Express skewness in terms of non-central (see Wikipedia for skewness)
	replace `gen'=normal(-1*`h')*`gen' // Change to unconditional mean
}
end

capture program drop gen_splines_exptrnorm
// Generate expected values of cubic splines, based on cubed moment of truncated normal
// distribution (uses gen_exp_trnorm3 above)
program gen_splines_exptrnorm
syntax , var(string) norm_mean(string) norm_sd(string) knots(string) ///
	[genprefix(string) replace]
qui {
	if "`genprefix'"==""&"`replace'"=="" local genprefix Etrn
	local words=wordcount("`knots'")
	foreach num of numlist 1/`words' {
		local knot`num'=word("`knots'",`num')
	}
	capture drop `genprefix'_`var'_sp1
	gen `genprefix'_`var'_sp1=`norm_mean'
	foreach num of numlist 2/`=`words'-1' {
		foreach dropvar in `genprefix'_`var'_sp`num' Etrn3_? {
			capture drop `dropvar'
		}
		gen_exp_trnorm3, mean(`norm_mean'-`knot`=`num'-1'') sd(`norm_sd') gen(Etrn3_1)
		gen_exp_trnorm3, mean(`norm_mean'-`knot`=`words'-1'') sd(`norm_sd') gen(Etrn3_2)
		gen_exp_trnorm3, mean(`norm_mean'-`knot`words'') sd(`norm_sd') gen(Etrn3_3)
		gen `genprefix'_`var'_sp`num'=(Etrn3_1-(`knot`words''-`knot`=`words'-1'')^-1* ///
			(Etrn3_2*(`knot`words''-`knot`=`num'-1'')-Etrn3_3*(`knot`=`words'-1''- ///
			`knot`=`num'-1'')))/(`knot`words''-`knot1')^2
		if "`replace'"=="replace" {
			replace `var'_sp`num'=`var'_sp`num'+`genprefix'_`var'_sp`num'
			drop `genprefix'_`var'_sp`num'
		}
		foreach dropvar in Etrn3_? {
			drop `dropvar'
		}
	}
}
end

capture program drop gen_arrival_outcomes_census
program gen_arrival_outcomes_census
syntax [if], interval(integer) arrival_estimates(string) xbd_lntdc_estimates(string) ///
	[cf_from(real 0) cf_extend(real 0) incensus visits1h]
qui {	
	if "`if'"!="" keep `if'
	if "`visits1h'"=="visits1h" local visits_vars visits1h visits1h_1
	keep nshiftid at_t irelto_base ed_hh ed_ym ed_dow nshifttype d_lntdc nroom ///
		census `visits_vars' reldc_t _merge `incensus'
	expand 2
	by nshiftid at_t, sort: replace census=census-1 if _n==1
	if "`incensus'"!="" by nshiftid at_t: replace incensus=incensus-1 if _n==1
	by nshiftid at_t: gen reldc_t2_h=min(cond(_n==1,1,reldc_t+1)*${interval}/60,2)

	// Counterfactual arrivals, borrowed from run_sim.ado
	if `cf_from'>0&`cf_extend'!=0 {
		local cf_from=ceil(`cf_from'*60/`interval')*`interval'/60
		local cf_extend=ceil(`cf_extend'*60/`interval')*`interval'/60
		if `cf_extend'>0 ///
			gen delta=max(0,min(`cf_from'-irelto_base*`interval'/60,`cf_extend'))* ///
				60/`interval'
		else {
			local rampup=1 // Note: this could be turned into an argument
			gen scale=cond(`cf_from'<=irelto_base*`interval'/60,0, ///
				cond(`cf_from'-`rampup'>=irelto_base*`interval'/60,1, ///
				ceil((`cf_from'-irelto_base*`interval'/60)/`rampup'*60/`interval')* ///
				`interval'/60))
			gen delta=scale*max((1-irelto_base)*`interval'/60,`cf_extend')*60/`interval'
			drop scale
		}
		gen irelto_base_orig=irelto_base
		replace irelto_base=irelto_base+delta
		drop delta
	}
	
	make_spline, retrieve estfile(`arrival_estimates')
	estimates use "${Estimates}/`arrival_estimates'"
	predict p1 p2 p3 p4 // ordered logit
	estimates use "${Estimates}/`xbd_lntdc_estimates'"
	local se=e(rmse)
	make_spline, retrieve estfile(`xbd_lntdc_estimates')
	predict mean_xbd_lntdc if _merge==3, xb
	gen_splines_exptrnorm, var(xbd_lntdc) norm_mean(mean_xbd_lntdc) norm_sd(`se') ///
		genprefix(EW_arr) knots(${xbd_lntdc_knots})
	gen_splines_exptrnorm, var(diff_lntdc) ///
		norm_mean(mean_xbd_lntdc-ln(.5*`interval'/60)) ///
		norm_sd(`se') genprefix(EW_arr) knots(${diff_lntdc_knots})
	if `cf_from'>0&`cf_extend'!=0 {
		replace irelto_base=irelto_base_orig
		drop irelto_base_orig
	}
	gen irelto_base_t2=irelto_base-1
	if "${irelto_base_knots}"!="" make_spline, prefix(EW_arr_irelto_base_sp) ///
		var(irelto_base_t2) knots(${irelto_base_knots})
	foreach var of varlist EW_arr_*_sp? p2-p4 {
		replace `var'=0 if _merge==2 // no arrivals after EOS
	}
	replace p1=1 if _merge==2
	keep nshiftid at_t census EW_arr_*_sp? `visits_vars' p? reldc_t2_h `incensus'

	// EW_reldc_sp? splines
	if "${reldc_h_knots}"!="" make_spline, prefix(EW_reldc_h_sp) ///
		var(reldc_t2_h) knots(${reldc_h_knots})
	
	// Scale up splines by expected number of arrivals
	foreach var of varlist EW_arr_*_sp? {
		replace `var'=`var'*(p2*1+p3*2+p4*3)
	}
	
	// State variables that depend more specifically on new arrivals
	expand 4 // At most 4 arrivals from the ordered logit model
	by nshiftid at_t census, sort: gen arrivals=_n-1
	gen newcensus=census+arrivals
	if "${census_knots}"!="" mkspline census_sp=newcensus, cubic ///
		knots(${census_knots})
	if "`incensus'"!="" {
		gen newincensus=incensus+arrivals
		if "${census_knots}"!="" mkspline incensus_sp=newincensus, cubic ///
			knots(${census_knots})
		local incensus_sp incensus_sp?
		local newincensus newincensus
	}
	if "${visits1h_knots}"!=""&"`visits1h'"=="visits1h" {
		replace visits1h=visits1h-visits1h_1+arrivals
		mkspline visits1h_sp=visits1h, cubic knots(${visits1h_knots})
	}
	gen prarrivals=cond(arrivals==0,p1,cond(arrivals==1,p2, ///
		cond(arrivals==2,p3,p4)))
	foreach var in census_sp? visits1h_sp? `incensus_sp' {
		capture d `var'
		if _rc==0 {
			foreach realvar of varlist `var' {
				by nshiftid at_t census: ///
					egen EW_`realvar'=total(prarrivals*`realvar')
				drop `realvar'
			}
		}
	}
	drop prarrivals arrivals newcensus `visits_vars' p? reldc_t2_h ///
		`newincensus'
	duplicates drop	
}
end

*** Main program *************************************************************************
capture program drop prep_dp_data
program prep_dp_data
syntax, [discharge_estimates(string) template(string) arrival_estimates(string) ///
	xbd_estimates(string) var_nknots(string) if(string) noknots nogendata ///
	nshifttypes(string) keepvars(string) nodcpr incensus ///
	keepirelto_range(string) visits1h ///
	cf_from(real 0) cf_extend(real 0)]
if c(noisily) local qui qui
`qui' {
	if "`knots'"=="noknots" {
		if "`var_nknots'"!="" {
			disp as error "Error: cannot specify -var_nknots- and -noknots-"
			exit
		}
		disp as text "Using precalculated knots:"
		foreach var in xbd_lntdc diff_lntdc adjirelto {		
			if "${`var'_knots}"=="" {
				disp as error "Error: -`var'_knots- unknown"
				exit
			}
		}
	}
	else {
		if "`var_nknots'"=="" {
			disp as error "Error: must specify -var_nknots-"
			exit
		}
		else foreach var in xbd_lntdc diff_lntdc adjirelto {
			if !regexm("`var_nknots'","`var'") {
				disp as error "Error: -`var'_knots- unknown"
				exit
			}
		}
	}
	if "`dcpr'"=="nodcpr" {
		if "`template'"==""|"`arrival_estimates'"==""|"`xbd_estimates'"=="" {
			noi disp as error "Error: must specify -template-, " ///
				"-arrival_estimates-, and -xbd_estimates-"
			exit
		}		
	}
	else {
		if "`discharge_estimates'"==""|"`template'"==""|"`arrival_estimates'"==""| ///
			"`xbd_estimates'"=="" {
			noi disp as error "Error: must specify -discharge_estimates-, " ///
				"-template-, -arrival_estimates-, and -xbd_estimates-"
			exit
		}		
	}
	if "`keepirelto_range'"!="" {
		if "`gendata'"=="nogendata" {
			disp as error "Error: -keepirelto_range- cannot be specified with " ///
				"-nogendata-"
			exit
		}
		capture assert wordcount("`keepirelto_range'")==2
		local error=_rc!=0
		tokenize "`keepirelto_range'"
		capture assert `1'<`2'
		local error=_rc!=0
		if !`error' {
			local min_keepirelto=`1'
			local max_keepirelto=`2'
		}
		else {
			disp as error "Error: invalid -keepirelto_range-"
			exit
		}		
	}
	if "`visits1h'"=="" {
		foreach nshifttype of numlist `nshifttypes' {
			estimates use "${Estimates}/`arrival_estimates'_`nshifttype'"
			estimates describe
			if regexm("`r(cmdline)'","visits1h") local visits1h visits1h		
		}
	}
	if "`visits1h'"=="visits1h" local visits_vars visits1h visits1h_1	
	
	tempfile working_sample W_splines EW_splines
	if "`gendata'"=="" {
		use "${Data}/stored_dp_data.dta", clear			
		merge m:1 nshiftid using "${Data}/nshiftid_sample_xwalk.dta", ///
			keep(match master) 
			// to reduce burden, select 10% of shifts if shiftlen=9
		keep if shiftlen!=9|_merge==3
		drop _merge
		if "`if'"!="" keep if `if'
		make_logit_dta, interval(${interval}) keepvars(lntdc t_ed_dcord) `visits1h'
		local keepvars `keepvars' choice
		gen reldc_t=at_t-lastdc_t
		gen reldc_h=min(reldc_t*${interval}/60,2) 
	}
	else egen groupid=group(nshiftid at_t)
	if "`incensus'"!="" {
		capture d sample 
		if _rc==0 by groupid, sort: egen incensus=total(sample==1)
		else by groupid, sort: gen incensus=_N-1
		local censusvar incensus
	}
	else local censusvar census

	if "`dcpr'"=="" {
		if "`gendata'"=="nogendata" {
			foreach var in irelto_base adjirelto census diff_lntdc xbd_lntdc ///
				hr_ed_hh sb dc {
				capture replace `var'=0 if sb==0
			}
			capture gen byte nodc=sb==0
		}
		gen reldc=cond(nodc,0,at_t-lastdc_t)
		make_spline, estfile(`discharge_estimates') retrieve
		estimates use "${Estimates}/`discharge_estimates'"
		predict pr
		local keepvars `keepvars' pr
		drop adjirel? cen? diff? baseirel? reldc_sp? reldc
	}

	// Generate working sample
	sort groupid
	foreach var of varlist irelto census adjirelto {
		by groupid: egen max_`var'=max(cond(sb!=0,`var',.))
		assert `var'==max_`var' if sb!=0
		replace `var'=max_`var' if sb==0
		drop max_`var'
		// Fixes issue of different values for sb==0
	}
	if "`keepirelto_range'"!="" keep if irelto<=`max_keepirelto'& ///
		irelto>=`min_keepirelto'
	if "`knots'"=="" { // Generate knots
		tokenize `var_nknots'	
		noi disp as text "Creating knots ..."
		foreach var in xbd_lntdc diff_lntdc irelto_base reldc_h `visits1h' census ///
			adjirelto {
			global `var'_knots
		}
		foreach token of numlist 1/`=wordcount("`var_nknots'")' {
			noi gen_spline_knot_macro, var_nknots(``token'') disp
		}
	}
	mkspline u_diff_lntdc_sp=diff_lntdc, knots(${diff_lntdc_knots}) cubic
	mkspline u_xbd_lntdc_sp=xbd_lntdc, knots(${xbd_lntdc_knots}) cubic
	if "${reldc_h_knots}"!="" make_spline `if_reldc_h', prefix(u_reldc_h_sp) ///
		var(reldc_h) knots(${reldc_h_knots})
	if "${irelto_base_knots}"!="" {
		make_spline, prefix(irelto_base_sp) ///
			var(irelto_base) knots(${irelto_base_knots})
		local irelto_splines irelto_base_sp?
		local EW_irelto_splines EW_irelto_base_sp?
	}
	foreach var of varlist u_* {
		if !inlist("`var'","u_posteos","u_posteos_t") replace `var'=0 if sb==0
	}
	save `working_sample', replace
	
	// W in splines
	if "`_wsplines'"=="" {
		use `working_sample', clear
		drop if sb==0
		sort nshiftid at_t 
		foreach var of varlist u_diff_lntdc_sp? u_xbd_lntdc_sp? `irelto_splines' {
			local root `=regexr("`var'","u_","")'
			by nshiftid at_t: egen W_`root'=total(`var')
		}
		keep nshiftid at_t W_* reldc_h census adjirelto `visits1h' irelto `incensus'
		duplicates drop
		foreach var in adjirelto census `visits1h' reldc_h {
			if "${`var'_knots}"!="" make_spline, prefix(W_`var'_sp) ///
				var(`var') knots(${`var'_knots})
		}
		if "`incensus'"!=""&"${census_knots}"!="" ///
			make_spline, prefix(W_incensus_sp) var(incensus) knots(${census_knots})
		drop reldc_h census adjirelto `visits1h' `incensus'
		gen W_cons=1
		save `W_splines', replace		
	}

	// EW in splines (conditional on sb discharge decision)
	use `working_sample', clear
	gen irelto_t2=irelto-1
	gen adjirelto_t2=adjirelto-1
	gen sim_lntdc_t2=ln((period+1)/60*${interval})
	gen diff_lntdc_t2=cond(sb==0,0,sim_lntdc_t2-xbd_lntdc)
	make_spline, prefix(diff_lntdc_sp) var(diff_lntdc_t2) knots(${diff_lntdc_knots})
	if "${adjirelto_knots}"!="" make_spline, prefix(EW_adjirelto_sp) ///
		var(adjirelto_t2) knots(${adjirelto_knots})
	sort nshiftid at_t
	foreach var of varlist diff_lntdc_sp? u_xbd_lntdc_sp? `irelto_splines' {
		local root `=regexr("`var'","u_","")'
		by nshiftid at_t: egen EW_`root'=total(`var')
		replace EW_`root'=EW_`root'-`var'
	}
	if "${reldc_h_knots}"!="" {
		gen reldc_t2_h=min(cond(sb!=0,1,reldc_t+1)*${interval}/60,2)
		make_spline, prefix(EW_reldc_h_sp) ///
			var(reldc_t2_h) knots(${reldc_h_knots})
	}		
	replace census=census-1 if sb!=0
	if "`incensus'"!="" replace incensus=incensus-1 if sb!=0
	gen EW_cons=1
	keep nshiftid at_t sb EW_* census irelto_t2 `incensus' `visits_vars'
	save `EW_splines', replace

	// Add on arrival outcomes for subsequent period, conditional on census post discharge
	// Only relevant if not at or past EOS
	sum irelto_t2 
	if r(max)>0 {
		use `working_sample', clear
		tempfile tagged_times
		drop if sb==0
		keep nshiftid at_t census `visits_vars' reldc_t nshifttype irelto `incensus'
		duplicates drop
		save `tagged_times', replace
		use "`template'", clear
		if "`if'"!="" keep if `if'
		gen at_t=ed_t-1 // at_t is discharge time, arrivals are in subsequent period ed_t
		merge 1:1 nshiftid at_t using `tagged_times', keep(match using)
		tempfile arrival_outcomes
		foreach nshifttype of numlist `nshifttypes' {
			count if nshifttype==`nshifttype'
			if r(N)>0 {
				preserve
				gen_arrival_outcomes_census if nshifttype==`nshifttype', ///
					arrival_estimates(`arrival_estimates'_`nshifttype') ///
					xbd_lntdc_estimates(`xbd_estimates') interval(${interval}) ///
					`incensus' `visits1h' ///
					cf_from(`cf_from') cf_extend(`cf_extend')
				capture append using `arrival_outcomes'
				save `arrival_outcomes', replace
				restore
			}
		}
		use `arrival_outcomes', clear
		merge 1:m nshiftid at_t census using `EW_splines', nogen
		foreach var of varlist EW_xbd_lntdc_sp? EW_diff_lntdc_sp? `EW_irelto_splines' {
			local root `=regexr("`var'","EW_","")'
			replace EW_`root'=EW_`root'+EW_arr_`root'
			drop EW_arr_`root'
		}
	}
	else {	
		use `EW_splines', clear		
		if "${census_knots}"!="" {
			mkspline EW_census_sp=census, cubic knots(${census_knots})
			if "`incensus'"!="" mkspline EW_incensus_sp=incensus, cubic ///
					knots(${census_knots})
		}
		if "${visits1h_knots}"!=""&"`visits1h'"=="visits1h" {
			replace visits1h=visits1h-visits1h_1
			mkspline EW_visits1h_sp=visits1h, cubic knots(${visits1h_knots})
		}
		capture drop `visits_vars'
	}
	drop census `incensus'
	save `EW_splines', replace

	// Combine datasets, with assertions
	use `working_sample', clear
	keep nshiftid at_t sb irelto u_* groupid `keepvars'
	merge 1:1 nshiftid at_t sb using `EW_splines'
	assert _merge==3
	drop _merge
	if "`_wsplines'"=="" {
		merge m:1 nshiftid at_t using `W_splines'
		assert _merge==3
		drop _merge
		local checkw W_*
	}
	foreach var of varlist nshiftid at_t sb u_* EW_* `checkw' `keepvars' {
		disp as text "`var'"
		if "`gendata'"=="" assert `var'!=.
		else if !regexm("`keepvars'","`var'") {
			assert `var'!=.|!sample `wtcond'
		}
		else if !inlist("`var'","reldc_t","lastdc_t","lntdc","lntdc_ptadj") {
			assert `var'!=.|!sample|sb<=0 `wtcond'
		}
	}
	gen byte u_dc=sb!=0
	foreach var of varlist u_* {
		disp as text "`var'"
		if !inlist("`var'","u_posteos","u_posteos_t") assert `var'==0 if sb==0
	}
	if "`_wsplines'"=="" {
		assert irelto_t2==irelto-1
		foreach var of varlist W_* {
			confirm var E`var'
		}
		foreach var of varlist EW_* {
			confirm var `=regexr("`var'","EW_","W_")'
		}
	}	
	format sb %12.0g
	sort groupid sb
	compress
}
end
