/*****************************************************************************************
AUTHOR: David Chan
PURPOSE: Main program to run simulations, including potentially simulating arrivals 
(sim_arrivals_multi.ado), expected length of stay of simulated arrivals 
(sim_xb_lntdc_multi), and discharges in counterfactuals
OPTIONS:
- dpirelto_range: Range between which discharges are simulated by Barwick-Pathak 
  parameters and value functions
- simirelto_range: Range between which discharges are simulated. This should be a superset
  of -dpirelto_range-; anything in this range and not in -dpirelto_range- will be 
  discharged in simulation using regular conditional logit probabilities 
  (-discharge_estimates-)
******************************************************************************************/

capture program drop run_sim
program run_sim
syntax [if], template(string) simirelto_range(string) [ext_estimates_opts(string) ///
	prep_dp_data_opts(string) discharge_estimates(string) arrival_estimates(string) ///
	xb_lntdc_estimates(string) interval(integer 60) reps(integer 1) noseed ///
	cf_from(real 0) cf_extend(real 0) cfonly scale_adjirelto(real 1) dpirelto_range(string)]
set more off
foreach range in simirelto dpirelto {
	if "``range'_range'"!="" {
		capture assert wordcount("``range'_range'")==2
		if _rc!=0 {
			disp as error "Error: invalid `range'"
			exit
		}
		tokenize "``range'_range'"
		capture assert `1'<=`2'|(`1'==1&`2'==0)
		local error=_rc!=0
		local min_`range'=`1'
		local max_`range'=`2'
		if `error' {
			disp as error "Note: possible error in `range'_range"
			exit // Note: may specify `1'==1 and `2'==0 if you want to bypass simulation
		}
	}
}
if "`dpirelto_range'"!=""& ///
	(`min_dpirelto'<`min_simirelto'|`max_dpirelto'>`max_simirelto') {
	disp as error "Error: simirelto_range not superset of dpirelto_range"
	exit
}
else if "`dpirelto_range'"=="" {
	local min_dpirelto=`min_simirelto'
	local max_dpirelto=`max_simirelto'
}
if "`dpirelto_range'"!="1 0" {
	if ("`ext_estimates_opts'"==""|"`prep_dp_data_opts'"=="") {
		disp as error "Error: -ext_estimates_opts- and -prep_dp_data_opts- required"
		exit
	}
}
local min_irelto=min(`min_simirelto',-4*60/${interval})

if "`cfonly'"=="cfonly" {
	local min_simarrival=1
	local max_simarrival=`cf_from'*60/`interval'-1
}
else if "`min_simarrival'"==""&"`max_simarrival'"=="" {
	local min_simarrival=max(1,`min_simirelto')
	local max_simarrival=`max_simirelto'
}
if `cf_from'>0&`cf_extend'!=0 {
	if (1<`min_simirelto'|`cf_from'*60/`interval'>`max_simirelto')| ///
	(1<`min_simarrival'|`cf_from'*60/`interval'-1>`max_simarrival'){
		disp as error "Error: counterfactual range incompatible"
		exit	
	}
}

qui {
	tempfile on_census outqueue discharged hour_census working_template master outvisits
	noi disp as text "Simulating arrivals and discharges `if'"
	if "`seed'"=="" set seed 0
	else {
		local str `=subinstr("`c(current_time)'",":","",.)'
		foreach pos of numlist 1/`=length("`str'")' {
			local newstr `newstr'`=substr("`str'",-`pos',1)'
		}
		set seed `=real("`newstr'")'
	}
	preserve

	if "`arrival_estimates'"==""|"`xb_lntdc_estimates'"=="" {
		if `cf_from'>0|`cf_extend'!=0 {
			noi disp as error ///
				"  Error: Cannot do counterfactuals if arrival_estimates or " ///
				"xb_lntdc_estimates unspecified"
			exit
		}
		else {
			noi disp as text "  Note: Taking all actual arrivals as given"
			local cfonly cfonly
		}
	}
	if "`cfonly'"=="cfonly"&!(`cf_from'>0&`cf_extend'!=0) {
		noi disp as text "  Note: equivalent to dconly"
	}	
	foreach range in simarrival simirelto dpirelto {
		if `min_`range''<=`max_`range'' noi disp as text "  `range'_range: " ///
			as result _col(21) "`min_`range'' `max_`range''"	
		else noi disp as text "  `range'_range: " ///
			as result _col(21) "null"		
	}
	
	// Check if need to keep track of visits
	local visits1h 
	if "`arrival_estimates'"!="" {
		estimates use "${Estimates}/`arrival_estimates'"
		estimates describe
		if regexm("`r(cmdline)'","visits1h") local visits1h visits1h
	}
	if "`visits1h'"==""&"`dpirelto_range'"!="1 0" {
		use "${estfile}", clear
		keep if estname=="${estname}":estname
		decode nvar, gen(var)
		count if regexm(var,"visits1h")
		if r(N)>0 local visits1h visits1h
	}
		
	// Prepare working_template 
	if "`cfonly'"==""|(`cf_from'>0&`cf_extend'!=0) {
		use "`template'", clear
		if "`if'"!="" keep `if'
		if `cf_from'>0&`cf_extend'!=0 {
			local cf_from=ceil(`cf_from'*60/`interval')*`interval'/60
			local cf_extend=ceil(`cf_extend'*60/`interval')*`interval'/60
			if `cf_extend'>0 ///
				gen delta=max(0,min(`cf_from'-irelto_base*`interval'/60,`cf_extend'))* ///
					60/`interval'
			else {
				local rampup=1 // Note: this could be turned into an argument
				gen scale=cond(`cf_from'<=irelto_base*`interval'/60,0, ///
					cond(`cf_from'-`rampup'>=irelto_base*`interval'/60,1, ///
					ceil((`cf_from'-irelto_base*`interval'/60)/`rampup'*60/`interval')* ///
					`interval'/60))
				gen delta=scale*max((1-irelto_base)*`interval'/60,`cf_extend')*60/`interval'
				drop scale
			}
			gen irelto_base_cf=irelto_base+delta
			gen irelfrom_base_cf=irelfrom_base-delta
		}
		save `working_template', replace	
	}
	
	restore // Back to observations created by "Load base data"
	preserve
	if "`if'"!="" keep `if'
	gen period=0
	gen at_t=ed_t
	gen irelto=.
	gen adjirelto=.
	gen sim_lntdc=.
	gen diff_lntdc=.	
	gen reldc_t=.
	gen reldc_h=.
	gen lastdc_t=.	
	gen cum_censushx=0
	gen hr_ed_hh=ed_hh
	save `outqueue', replace
	sum irelfrom_base
	local irelfrom_base=r(min) 
		// Usually determined by -posthrs()- in gen_reltimeext.ado creating data
	local max_irelfrom_base=r(max)
	local min_min_irelfrom_base=`irelfrom_base'
	local N_outqueue=0
	local N_census=0
	local N_discharged=0
	local min_pseudosb=0
	scalar sc_min_min_irelfrom_base=`min_min_irelfrom_base'
	scalar sc_irelfrom_base=`irelfrom_base'
	scalar sc_irelto=0
	scalar sc_N_census=`N_census'
	scalar sc_N_outqueue=`N_outqueue'
	scalar sc_N_discharged=`N_discharged'
	scalar sc_counter=0
	local begtime clock("1jun2005 00:00","DMYhm")
	sum shiftlen
	assert r(min)==r(max) 
		// For now, this algorithm is only equipped for one shiftlen at a time
	local shiftlen=`r(min)'

	// Optionally set up visits template (only for non-simulated arrivals here)
	if "`visits1h'"=="visits1h" {
		tempfile visits visits1h_file visits_obs
		if !("`cfonly'"==""|(`cf_from'>0&`cf_extend'!=0)) {
			use "`template'", clear
			if "`if'"!="" keep `if'
			expand 1+60/`interval'
			gen at_t=ed_t
			by nshiftid at_t, sort: replace ed_t=at_t-_n+1
			save `visits', replace
			use `outqueue', clear
			by nshiftid ed_t, sort: egen visits=total(sample)
			by nshiftid ed_t: keep if _n==1
			keep nshiftid ed_t visits
			merge 1:m nshiftid ed_t using `visits', keep(using match) nogen
			replace visits=0 if visits==.
			sort nshiftid at_t ed_t
			by nshiftid at_t: gen byte visits1h_1=visits[1]
			by nshiftid at_t: egen byte visits1h=total(visits)
			keep nshiftid at_t visits1h visits1h_1
			by nshiftid at_t: keep if _n==1
			save `visits1h_file', replace		
		}
	}
}

qui while `irelfrom_base'<=max(1,`min_min_irelfrom_base')|`N_census'>0 {
	local irelto_base=60/`interval'*`shiftlen'-`irelfrom_base'+1
	local newoutobs=0
	local newobs=0
	use `outqueue', clear
	keep if irelfrom_base==`irelfrom_base'
	if ("`cfonly'"==""|(`cf_from'>0&`cf_extend'!=0)) {
		if (`irelto_base'>=`min_simarrival'&`irelto_base'<=`max_simarrival') ///
			keep if sample==0
		if _N>0 {
			save `master', replace
			by nshiftid, sort: gen outvisits=_N		
			keep nshiftid outvisits irelfrom_base ed_t
			by nshiftid: keep if _n==1
			save `outvisits', replace
			use `master', clear
			local newoutobs=1
		}
	}
	if `N_census'>0 append using `on_census'
	save `on_census', replace
	
	// Simulate arrivals
	if "`cfonly'"==""|(`cf_from'>0&`cf_extend'!=0) {
		if `irelto_base'>=`min_simarrival'&`irelto_base'<=`max_simarrival' {
			if `N_census'>0 {
				keep at_t census nshiftid
				drop if census==.
				duplicates drop
				egen totcensus=total(census)
				sum totcensus
				assert r(mean)==`N_census'
				drop totcensus
				save `hour_census', replace
			}
			capture use `visits1h_file', clear
			if _rc==0 local hasvisits1h_file=1
			else local hasvisits1h_file=0
			use `working_template', clear
			keep if irelfrom_base==`irelfrom_base'

			count
			if r(N)>0 {				
				if `N_census'>0 {
					merge 1:1 nshiftid using `hour_census', assert(match master) nogen
					replace census=0 if census==.
					// Note censuses are based on previous period
				}
				else gen census=0

				if `cf_from'>0&`cf_extend'!=0 {
					foreach var in irelfrom_base irelto_base {
						rename `var' `var'_orig
						rename `var'_cf `var'
					}
				}
				if "`visits1h'"=="visits1h" {
					if `hasvisits1h_file' {
						merge 1:1 nshiftid using `visits1h_file', nogen assert(match master)
						replace visits1h=0 if visits1h==.
					}
					else gen visits1h=0
					sim_arrivals_multi, estimates(`arrival_estimates')
					save `master', replace
					if `cf_from'>0&`cf_extend'!=0 replace irelfrom_base=irelfrom_base_orig
					keep nshiftid visits irelfrom_base ed_t
					if `newoutobs' {
						merge 1:1 nshiftid using `outvisits', assert(match master) nogen
						replace outvisits=0 if outvisits==.
						replace visits=visits+outvisits
						drop outvisits
					}
					if _N>0 local newobs=1
					save `visits_obs', replace
					use `master', clear
				}
				else sim_arrivals_multi, estimates(`arrival_estimates')			
				if `newoutobs' {
					merge 1:1 nshiftid using `outvisits', assert(match master) nogen
					gen allcensus=census+cond(outvisits==.,0,outvisits)
					drop outvisits
				}
				else gen allcensus=census
				assert allcensus!=.&visits!=.
				replace visits=max(0,min(visits,25-allcensus)) // empirically census never exceeds 23
				drop if visits==0
				expand visits				

				if `cf_from'>0&`cf_extend'!=0 {
					foreach var in irelfrom_base irelto_base {
						rename `var' `var'_cf
						rename `var'_orig `var'
					}
				}
				sim_xbd_lntdc_multi, estimates(`xb_lntdc_estimates') 
				gen sb=-_n+`min_pseudosb' // Pseudo-sb is negative number
				local min_pseudosb=-_N+`min_pseudosb'
				append using `on_census'
				save `on_census', replace
			}
			else use `on_census', clear
		}
		if "`visits1h'"=="visits1h"&`irelfrom_base'<=60/`interval'*`shiftlen' {
			local continue=0
			if `newobs' {
				use `visits_obs', clear
				capture append using `visits'
				assert visits!=.
			}
			else if `newoutobs' {
				use `outvisits', clear
				rename outvisits visits
				capture append using `visits'
				assert visits!=.
			}
			else {
				capture use `visits', clear
				if _rc==0&_N>0 local continue=1
			}
			if `continue'|`newobs' {
				drop if irelfrom_base<`irelfrom_base'-60/`interval'
				if _N>0 {
					save `visits', replace
					by nshiftid irelfrom_base, sort: assert _N==1
					by nshiftid: gen visits1h_1=visits[1]
					by nshiftid: egen visits1h=total(visits)
					keep nshiftid visits1h visits1h_1
					by nshiftid: keep if _n==1
					save `visits1h_file', replace
					local hasvisits1h_file=1
				}
				else local hasvisits1h_file=0
			}
			else local hasvisits1h_file=0
			use `on_census', clear
		}
	}
	assert sample==1&irelto_base>=`min_simarrival'& ///
		irelto_base<=`max_simarrival' if sb<0
	assert sample==0|sb<0 if irelto_base>=`min_simarrival'& ///
		irelto_base<=`max_simarrival'
	assert sb>0 if irelto_base<`min_simarrival'| ///
		irelto_base>`max_simarrival'

	if _N>0 {
		replace period=period+1
		replace at_t=ed_t+period-1	
		capture drop census
		by nshiftid, sort: egen sd_at_t=sd(at_t)
		assert sd_at_t==0|sd_at_t==.
		drop sd_at_t
		by nshiftid: egen census=count(_n) 
			// Generate crude census, c/w multinomial logit, arrival estimation, 
			// and prior run_sim.ado
		by nshiftid: egen minlastdc_t=min(lastdc_t)
		replace lastdc_t=minlastdc_t
		drop minlastdc_t
		replace reldc_t=at_t-lastdc_t
		replace reldc_h=min(reldc_t*`interval'/60,2)	

		// Discharge patients if after shift beginning
		if `irelfrom_base'>=1 {
			replace irelto=(irelto_base-(period-1)) 
			sum irelto
			local mean_irelto=`r(mean)'
			assert r(sd)==0|r(N)==1
			scalar sc_irelto=`mean_irelto'
			assert `mean_irelto'==`irelto_base'
			if "`simirelto_range'"!=""|"`dpirelto_range'"!="" {
				local sim=(`mean_irelto'>=`min_simirelto')& ///
					(`mean_irelto'<=`max_simirelto')
				local bp=(`mean_irelto'>=`min_dpirelto')& ///
					(`mean_irelto'<=`max_dpirelto')
			}
			else {
				local sim=1
				local bp=1
			}
			if "`min_irelto'"!="" local notpost=`mean_irelto'>=`min_irelto'
			else local notpost=1
			if `sim' {
				
				replace sim_lntdc=ln(cond(period==1,.5,period)/60*`interval')
				replace diff_lntdc=sim_lntdc-xbd_lntdc
				replace adjirelto=cond(irelto<5*60/`interval', ///
					ceil(5*60/`interval'-(5*60/`interval'-irelto)*`scale_adjirelto'), ///
					irelto)+12*60/`interval'-1				

				replace hr_ed_hh=hh(`begtime'+msofminutes(`interval'*at_t))

				save `master', replace
				keep nshiftid at_t nshifttype
				duplicates drop
				gen sb=0
				gen irelto_base=0
				gen byte sample=1
				append using `master'
				
				if !`bp' {
					local probj pr
					foreach var in irelto_base adjirelto census diff_lntdc xbd_lntdc ///
						hr_ed_hh sb dc {
						capture replace `var'=0 if sb==0
					}
					gen byte nodc=sb==0
					gen reldc=cond(nodc,0,at_t-lastdc_t)
					make_spline, estfile(`discharge_estimates') retrieve
					estimates use "${Estimates}/`discharge_estimates'"
					egen groupid=group(nshiftid at_t)
					predict pr
					drop adjirel? cen? diff? baseirel? reldc_sp? reldc
				}
				else {
					if "`visits1h'"=="visits1h" {
						if "`cfonly'"==""|(`cf_from'>0&`cf_extend'!=0) {
							if `hasvisits1h_file' {
								merge m:1 nshiftid using `visits1h_file', keep(match master) nogen
								replace visits1h=0 if visits1h==.
								replace visits1h_1=0 if visits1h_1==.
							}
							else {
								gen visits1h=0
								gen visits1h_1=0
							}
						}	
						else merge m:1 nshiftid at_t using `visits1h_file', ///
							keep(match) assert(match using) nogen
					}
					local probj expdc_xb
					prep_dp_data, nogendata noknots nodcpr ///
						keepvars(sample period ed_t lastdc_t xbd_lntdc census ///
						diff_lntdc sim_lntdc reldc_t reldc_h dcord_t irelto_base ///
						cum_censushx nshifttype irelfrom_base t_ed_dcord lntdc ///
						lntdc_ptadj avgcensus) `prep_dp_data_opts' `visits1h'
					ext_estimates, `ext_estimates_opts' predict(dc_xb)
					gen expdc_xb=cond(dc_xb<50,exp(dc_xb),exp(50))
				}
				replace `probj'=. if !sample
				sort nshiftid sb
				by nshiftid: gen upper=sum(`probj')
				if "`probj'"=="pr" by nshiftid: assert round(upper[_N]*1000)==1000
				by nshiftid: egen maxupper=max(upper)
				by nshiftid: replace upper=upper/maxupper
				by nshiftid: gen lower=cond(_n==1,0,upper[_n-1])
				
				gen rand=runiform()
				by nshiftid: replace rand=rand[1]
				replace lower=. if !sample
				assert sample==0|`probj'!=.
				gen byte to_discharge=cond(`probj'!=.,(rand<upper&rand>lower) ///
					|period>=12*60/`interval',dcord_t<=at_t)
			}
			else if `notpost' gen byte to_discharge=dcord_t<=at_t
			else gen byte to_discharge=1
			
			drop if sb==0
			by nshiftid, sort: egen dccount=total(sample==1&to_discharge ///
				&period<12*60/`interval')
			assert dccount<=1|!`sim'
			by nshiftid, sort: egen hasdc=max(to_discharge*sample)
			replace lastdc_t=at_t if hasdc		
			foreach var in condprobdc dc_xb expdc_xb rand *upper lower hasdc groupid ///
				irelto_t2 u_* EW_* W_* visits1h visits1h_1 dccount pr nodc {
				capture drop `var'
			}
						
			save `on_census', replace
			keep if to_discharge
			drop to_discharge
			if `N_discharged'>0 append using `discharged'
			save `discharged', replace
			count
			local N_discharged=r(N)
			use `on_census', clear
			drop if to_discharge
			drop to_discharge census
			capture by nshiftid: egen census=count(_n)
			capture confirm var incensus
			if _rc==0 assert incensus<=23
		}
	}
	count
	local N_census=r(N)
	save `on_census', replace
	use `outqueue', clear
	drop if irelfrom_base==`irelfrom_base'
	count
	local N_outqueue=r(N)
	save `outqueue', replace
	
	// Display progress
		if `irelfrom_base'==`min_min_irelfrom_base' {
			noi disp as text "Progress:"
			noi disp as text "        Arrivals: 0" _cont
			local oldprog=0
			local newprog=0
		}
		else if `irelfrom_base'<`max_irelfrom_base' ///
			local newprog=ceil((`irelfrom_base'-`min_min_irelfrom_base')/ ///
				`=`max_irelfrom_base'-`min_min_irelfrom_base''*40)
		else if `irelfrom_base'==`max_irelfrom_base' {
			noi disp 
			local N_census_index=`N_census'
			noi disp as text " Only discharges: 0" _cont
			local oldprog=0
			local newprog=0
		}
		else if `irelfrom_base'>`max_irelfrom_base' ///
			local newprog=ceil((`N_census_index'-`N_census')/`N_census_index'*40)
		if `newprog'>`oldprog' {
			foreach num of numlist `=`oldprog'+1'/`newprog' {
				if ceil(`num'/4)==`num'/4 noi disp as text "`=`num'/4'0" _cont
				else noi disp as text "." _cont
			}
			local oldprog=`newprog'
		}
	local irelfrom_base=`irelfrom_base'+1
	scalar sc_irelfrom_base=`irelfrom_base'
	scalar sc_N_census=`N_census'
	scalar sc_N_outqueue=`N_outqueue'
	scalar sc_N_discharged=`N_discharged'
	scalar sc_counter=sc_counter+1
}	

qui {
	use `discharged', clear
	restore, not
	count if sample
	local numsample=r(N)
	count if !sample
	local notsample=r(N)
	foreach var of varlist * { 
		// some variables in !`inrange' kept unintentially, get rid of these
		sum `var'
		if !(r(N)==`numsample'|r(N)==_N|r(N)==`notsample'| ///
			"`var'"=="dcord_t") drop `var'
	}		
}
end
