/*****************************************************************************************
AUTHOR: David Chan
PURPOSE: Simulate discharges, holding arrivals fixed
******************************************************************************************/

capture program drop run_sim_dconly
program run_sim_dconly
syntax [if], [interval(integer 60) reps(integer 1) norestore estimates(string) flexible]
	preserve
qui {
	noi disp as text "Simulating arrivals and discharges `if'"
	if "`estimates'"=="" {
		if "`flexible'"=="flexible" {
			noi disp as error "  Error: must provide estimates with -flexible- option"
			exit
		}
		else noi disp as text "  Note: No estimates provided"
	}
	else {
		noi disp as text "  Note: Using `estimates'"
		estimates use "${Estimates}/`estimates'"
	}

	set seed 0
	if "`if'"!="" keep `if'
	
	gen period=0
	gen at_t=ed_t
	gen adjirelto=.
	gen sim_lntdc=.
	gen diff_lntdc=.
	gen reldc=.
	gen lastdc_t=.
	gen cum_censushx=0
	gen hr_ed_hh=ed_hh
	tempfile on_census queue discharged 
	save `queue', replace
	sum irelfrom_base
	local irelfrom_base=r(min)
	local N_queue=0
	local N_census=0
	local N_discharged=0
	local begtime clock("1jun2005 00:00","DMYhm")
	local oldprog=0
	count
	local N_total=r(N)
	set more off	
}
qui while `irelfrom_base'<=1|`N_census'>0 {
	use `queue', clear
	keep if irelfrom_base==`irelfrom_base'
	if `N_census'>0 append using `on_census'
	replace period=period+1
	replace at_t=ed_t+period-1
	replace reldc=at_t-lastdc_t
	replace sim_lntdc=ln(cond(period==1,.5,period)/60*`interval')
	replace diff_lntdc=sim_lntdc-xbd_lntdc
	gen wt=cond(period==1,.5,1)
	
	// Discharge patients if after shift beginning
	if `irelfrom_base'<1 {
		by nshiftid npid at_t, sort: egen census=total(wt) 
		replace cum_censushx=cum_censushx+census
		drop census wt
	}
	else {
		by nshiftid npid at_t, sort: egen census=count(_n) 
			// Generate crude census
		replace adjirelto=(irelto_base-(period-1))+12*60/`interval'-1
		replace hr_ed_hh=hh(`begtime'+msofminutes(`interval'*at_t))
		
		if "`flexible'"=="flexible" make_spline, retrieve estfile(`estimates')
		else make_splines, baseirel_var(irelto_base) adjirel_var(adjirelto)  ///
			per_var(period) cen_var(census) baseirel_numlist(2 4 6 9) ///
			adjirel_numlist(5 9 12 15 19) per_numlist(1/3 4 8) cen_numlist(3 7 12)
		
		predict dc_xb, xb
		gen rand=runiform()
		sort dc_xb // Otherwise there is randomness even when setting seed
		gen condprobdc=exp(dc_xb)/(exp(dc_xb)+1) if sample
		gen byte to_discharge=cond(sample,rand<condprobdc|period>12*60/`interval', ///
			dcord_t==at_t|period-irelto_base>12*60/`interval')
		by nshiftid npid at_t, sort: egen hasdc=max(to_discharge*sample)
		replace lastdc_t=at_t if hasdc		
		foreach var in baseirel? adjirel? per? cen? condprobdc dc_xb rand *upper lower ///
			diff_lntdc nodc hasdc {
			capture drop `var'
		}
		
		drop census
		replace wt=cond(period==1,.3,.5) if to_discharge
		by nshiftid npid at_t, sort: egen census=total(wt)
		replace cum_censushx=cum_censushx+census		
		drop wt census
		
		save `on_census', replace
		keep if to_discharge
		drop to_discharge
		if `N_discharged'>0 append using `discharged'
		save `discharged', replace
		count
		local N_discharged=r(N)
		use `on_census', clear
		drop if to_discharge
		drop to_discharge
	}

	count
	local N_census=r(N)
	save `on_census', replace
	use `queue', clear
	drop if irelfrom_base==`irelfrom_base'
	count
	local N_queue=r(N)
	save `queue', replace
	
	// Display progress
		if `irelfrom_base'==1 noi disp as text "Progress: 0" _cont
		else {
			local newprog=ceil(`N_discharged'/`N_total'*40)
			if `newprog'>`oldprog' {
				foreach num of numlist `=`oldprog'+1'/`newprog' {
					if ceil(`num'/4)==`num'/4 noi disp as text "`=`num'/4'0" _cont
					else noi disp as text "." _cont
				}
			}
			local oldprog=`newprog'
		}
	local irelfrom_base=`irelfrom_base'+1
}	

	if "`restore'"=="" restore
	else {
		use `discharged', clear
		restore, not
	}
end
