/*****************************************************************************************
AUTHOR: David Chan
PURPOSE: Load in scheduling data and change to time-interval format 
OPTIONS:
- posthrs: number of hours after a patient's arrival that a physician's shift can start
  (i.e., the patient arrives before the shift's start)
- prehrs: number of hours before a patient's arrival that a physician's shift can end
  (i.e., the patient arrives after the shifts start)
- interval: time interval in minutes
- save: save file (to save time on next run); when no name specified in -saveas-, the
  default name is choice_hour_obs
- saveas: name to save file as
- retrieve: retrieve from saved file (to save time); when no name specified, the default 
  name is choice_hour_obs
- retrievefrom: name of file to retrieve
******************************************************************************************/

capture program drop schedule_in_intervals
program schedule_in_intervals 
syntax , [posthrs(integer 0) prehrs(integer 0) save ///
	interval(integer 60) saveas(string) retrieve retrievefrom(string)]
		
	local fileok=1
	qui if "`retrieve'"!=""|"`retrievefrom'"!="" {
		if "`retrieve'"=="retrieve"&"`retrievefrom'"=="" ///
			local retrievefrom choice_hour_obs
		capture use "`retrievefrom'", clear
		if _rc!=0 {
			local fileok=0
			local invalid `retrievefrom' not found
		}
		if `fileok' foreach param in posthrs prehrs interval {
			local num=1
			local param_met=0
			while "`_dta[note`num']'"!=""&!`param_met' {
				local string `_dta[note`num']'
				if regexm("`string'","`param'=``param''") local param_met=1
				else local invalid `retrievefrom' invalid (`param'=``param'')
				local num=`num'+1
			}
			if !`param_met' local fileok=0
		}
		
		if `fileok' noi disp as text "  Note: using `retrievefrom' (verified)"
		else noi disp as error "  Note: `invalid'"
	}
	
	if ("`retrieve'"==""&"`retrievefrom'"=="")|!`fileok' qui {
		use "${Data}/schedule_data", clear
		name_pods, `keepecnu'
		gen_var, room
		
		// Process partial shifts
		replace totc=subtotc if subtotc!=.
		replace fromtc=subfromtc if subfromtc!=.
		replace to=subto if subto!=""
		replace from=subfrom if subfrom!=""
		
		// correct for back-to-back shifts
		sort npid pod fromtc
		by npid pod: replace totc=totc[_n+1] if fromtc[_n+1]!=.&fromtc[_n+1]<totc
		by npid pod: replace to=to[_n+1] if totc==totc[_n+1] 
		by npid pod: replace fromtc=fromtc[_n-1] if totc[_n-1]!=.&totc[_n-1]>fromtc
		by npid pod: replace from=from[_n-1] if fromtc==fromtc[_n-1] 
		duplicates drop npid pod totc fromtc, force
		cat_shift_types
		gen shiftlen=real(word(subinstr(shifttype,"_"," ",.),1)) 
		gen preovl=real(word(subinstr(shifttype,"_"," ",.),2)) 
		gen postovl=real(word(subinstr(shifttype,"_"," ",.),3))
		encode shifttype, gen(nshifttype)
		
		// capture dates and times
		gen hours=hours(totc-fromtc)
		drop if hours==.
		expand (hours+`posthrs'+`prehrs')*60/`interval'
		rename date origdate
		by origdate shift from, sort: ///
			gen double at_tc=fromtc+msofhours((_n-1)*`interval'/60-`posthrs' ///
			+`interval'/60/2)
		gen date=dofc(at_tc)
		gen hour=hh(at_tc)
		format date %td
		format at_tc %tc

		if `interval'==60 local timeid date hour
		else {
			local begtime clock("1jun2005 00:00","DMYhm")
			gen at_t=ceil(minutes(at_tc-`begtime')/`interval')
			local timeid at_t
			local keep at_t
		}
		
		// relative times (in hours)
		gen relfrom=hours(at_tc-fromtc)
		gen relto=hours(at_tc-totc)
		gen byte notpost=relfrom>=0 // beginning of shift not after this time
		gen byte notpre=relto<=0 // end of shift not prior to this time 
		
		// organize and identify shifts, rename variables
		duplicates drop
		by `timeid' pod, sort: egen byte truechoices=total(notpost&notpre)
		drop if npid==. // "open shifts" with no name in schedule
		decode nshiftid, gen(shiftid)
		replace shiftid=shiftid+" "+subfrom if subfrom!=""
		rename fromtc shiftbeg_tc
		rename totc shiftend_tc
		keep npid *shiftid shiftbeg_tc shiftend_tc date hour at_tc shift rel* ///
			room pod `keep' not* truechoices nshifttype shiftlen preovl postovl
	
		noi disp as text "  Deleting duplicates ..."
		drop if regexm(shift,"admin")|regexm(shift,"oncall")
		foreach time in post pre {
			duplicates tag `timeid' npid, gen(tagged)
			drop if tagged&!not`time'
			drop tagged
		}
		
		if ("`save'"=="save"|"`saveas'"!="")&!`fileok' {
			if "`saveas'"==""&"`save'"=="save" local saveas choice_hour_obs
			note: posthrs=`posthrs', prehrs=`prehrs'
			note: interval=`interval'; timeid: `timeid'
			save "${Data}/`saveas'", replace
			noi disp as text "  Saved file: `saveas'"
		}			
	}
end
